import json
from concurrent import futures

import grpc

from . import connection_pb2
from .connection_base import (ConnectionBase, ConnectionListenerBase,
                              ConnectionResourceListenerBase)
from .connection_pb2_grpc import (ConnectionGrpcServicer,
                                  add_ConnectionGrpcServicer_to_server)


class ConnectionGrpc(ConnectionBase, ConnectionGrpcServicer):
  def __init__(self, eventListener: ConnectionListenerBase, resourceListener: ConnectionResourceListenerBase):
    super().__init__(eventListener=eventListener, resourceListener=resourceListener)
    self.id = hex(id(self))

  # rpc GetNodeStructures(Empty) returns (GetNodeStructuresResponse) {}
  def GetNodeStructures(self, request, context):
    try:
      nodeStructures = self._getNodesStructuresEmit()
    except Exception as e:
      self._logger.exception('Exception occured')
      raise e
    return connection_pb2.GetNodeStructuresResponse(serverId=self.id, jsonData=json.dumps(nodeStructures) if nodeStructures else None)

  # rpc GetCurrentExistingNodes(Empty) returns (GetCurrentExistingNodesResponse) {}
  def GetCurrentExistingNodes(self, request, context):
    try:
      nodes = self._getCurrentExistingNodesEmit()
    except Exception as e:
      self._logger.exception('Exception occured')
      raise e
    return connection_pb2.GetCurrentExistingNodesResponse(serverId=self.id, jsonData=json.dumps(nodes) if nodes is not None else None)

  # rpc CreateNodeInstance(CreateNodeInstanceRequest) returns (CreateNodeInstanceResponse) {}
  def CreateNodeInstance(self, request, context):
    try:
      result, errorMessage = self._createNodeEmit(request.id, request.nodePath, request.name)
    except Exception as e:
      self._logger.exception('Exception occured')
      raise e
    return connection_pb2.CreateNodeInstanceResponse(serverId=self.id, result=result, errorMessage=errorMessage)

  # rpc UpdatePortValues(UpdatePortValuesRequest) returns (UpdatePortValuesResponse) {}
  def UpdatePortValues(self, request, context):
    portValues = json.loads(request.portValuesJson) if request.portValuesJson else None
    try:
      result, errorMessage = self._updatePortValues(request.id, portValues)
    except Exception as e:
      self._logger.exception('Exception occured')
      raise e
    return connection_pb2.UpdatePortValuesResponse(serverId=self.id, result=result, errorMessage=errorMessage)

  # rpc ExecuteNode(ExecuteNodeRequest) returns (ExecuteNodeResponse) {}
  def ExecuteNode(self, request, context):
    portValues = json.loads(request.inputPortValuesJson) if request.inputPortValuesJson else None
    try:
      result, errorMessage, locals, outputs = self._nodeExecutionEmit(request.id, request.portName, portValues)
    except Exception as e:
      self._logger.exception('Exception occured')
      raise e
    return connection_pb2.ExecuteNodeResponse(serverId=self.id, result=result, errorMessage=errorMessage, localPortValuesJson=json.dumps(locals) if locals else None, outputPortValuesJson=json.dumps(outputs) if outputs else None)

  # rpc RemoveNode(RemoveNodeRequest) returns (RemoveNodeResponse) {}
  def RemoveNode(self, request, context):
    try:
      result, errorMessage = self._removeNodeEmit(request.id)
    except Exception as e:
      self._logger.exception('Exception occured')
      raise e
    return connection_pb2.RemoveNodeResponse(serverId=self.id, result=result, errorMessage=errorMessage)

  # rpc SendResource(stream SendResourceRequest) returns (SendResourceResponse) {}
  def SendResource(self, request, context):
    try:
      result = self._resourceReceivedEmit(request.metadata.id, request.metadata.type, request.data.content)
    except Exception as e:
      self._logger.exception('Exception occured')
      raise e
    return connection_pb2.SendResourceResponse(serverId=self.id, result=result)

  # rpc GetResource(GetResourceRequest) returns (stream GetResourceResponse) {}
  def GetResource(self, request, context):
    try:
      id_, type_, data = self._getResourceEmit(request.id, request.remove)
    except Exception as e:
      self._logger.exception('Exception occured')
      raise e
    return connection_pb2.GetResourceResponse(serverId=self.id, metadata=connection_pb2.ResourceMetaData(id=id_, type=type_), data=connection_pb2.ResourceData(content=data))

  # rpc ReleaseResource(ReleaseResourceRequest) returns (ReleaseResourceResponse) {}
  def ReleaseResource(self, request, context):
    try:
      result, errorMessage = self._releaseResourceEmit(request.id)
    except Exception as e:
      self._logger.exception('Exception occured')
      raise e
    return connection_pb2.ReleaseResourceResponse(serverId=self.id, result=result, errorMessage=errorMessage)

  @classmethod
  def createServerInstance(cls, eventListener=None, resourceListener=None, port=50051):
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=10))
    add_ConnectionGrpcServicer_to_server(ConnectionGrpc(eventListener, resourceListener), server)
    server.add_insecure_port(f'[::]:{port}')
    return server





from .node_base import NodeBase
from .exceptions import NodeRegistrationError, NodeNotFoundError

class NodeFactory(object):
  def __init__(self) -> None:
    super().__init__()
    self._nodes = {}
    self._names = {}

  def registerNode(self, node: NodeBase):
    if node is None:
      raise ValueError('node param is invalid')

    name = node.NODE_NAME
    node_path = node.getNodePath()

    if name in self._names:
      raise NodeRegistrationError(f'Node name "{name}" is already registered')

    if self._nodes.get(node_path.lower()):
      raise NodeRegistrationError(f'Node "{node_path}" is already registered')

    self._nodes[node_path.lower()] = node
    self._names[name] = node_path.lower()

  def getNodesStructures(self) -> list:
    result = []
    for identifier, node in self._nodes.items():
      result.append(node.getNodeStructure())
    return result

  def getNodeClass(self, path) -> NodeBase:
    if not self.isPathValid(path):
      raise ValueError('invalid path')

    nodeClass = self._nodes.get(path.lower(), None)
    if not nodeClass:
      raise NodeNotFoundError(f'Node {path} was not found')
    return nodeClass

  def isPathValid(self, path: str):
    if not path:
      return False

    return path.find(' ') == -1



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from . import connection_pb2 as connection__pb2


class ConnectionGrpcStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.GetNodeStructures = channel.unary_unary(
                '/connection.ConnectionGrpc/GetNodeStructures',
                request_serializer=connection__pb2.Empty.SerializeToString,
                response_deserializer=connection__pb2.GetNodeStructuresResponse.FromString,
                )
        self.GetCurrentExistingNodes = channel.unary_unary(
                '/connection.ConnectionGrpc/GetCurrentExistingNodes',
                request_serializer=connection__pb2.Empty.SerializeToString,
                response_deserializer=connection__pb2.GetCurrentExistingNodesResponse.FromString,
                )
        self.CreateNodeInstance = channel.unary_unary(
                '/connection.ConnectionGrpc/CreateNodeInstance',
                request_serializer=connection__pb2.CreateNodeInstanceRequest.SerializeToString,
                response_deserializer=connection__pb2.CreateNodeInstanceResponse.FromString,
                )
        self.UpdatePortValues = channel.unary_unary(
                '/connection.ConnectionGrpc/UpdatePortValues',
                request_serializer=connection__pb2.UpdatePortValuesRequest.SerializeToString,
                response_deserializer=connection__pb2.UpdatePortValuesResponse.FromString,
                )
        self.ExecuteNode = channel.unary_unary(
                '/connection.ConnectionGrpc/ExecuteNode',
                request_serializer=connection__pb2.ExecuteNodeRequest.SerializeToString,
                response_deserializer=connection__pb2.ExecuteNodeResponse.FromString,
                )
        self.RemoveNode = channel.unary_unary(
                '/connection.ConnectionGrpc/RemoveNode',
                request_serializer=connection__pb2.RemoveNodeRequest.SerializeToString,
                response_deserializer=connection__pb2.RemoveNodeResponse.FromString,
                )
        self.SendResource = channel.unary_unary(
                '/connection.ConnectionGrpc/SendResource',
                request_serializer=connection__pb2.SendResourceRequest.SerializeToString,
                response_deserializer=connection__pb2.SendResourceResponse.FromString,
                )
        self.GetResource = channel.unary_unary(
                '/connection.ConnectionGrpc/GetResource',
                request_serializer=connection__pb2.GetResourceRequest.SerializeToString,
                response_deserializer=connection__pb2.GetResourceResponse.FromString,
                )
        self.ReleaseResource = channel.unary_unary(
                '/connection.ConnectionGrpc/ReleaseResource',
                request_serializer=connection__pb2.ReleaseResourceRequest.SerializeToString,
                response_deserializer=connection__pb2.ReleaseResourceResponse.FromString,
                )


class ConnectionGrpcServicer(object):
    """Missing associated documentation comment in .proto file."""

    def GetNodeStructures(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetCurrentExistingNodes(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateNodeInstance(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UpdatePortValues(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ExecuteNode(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RemoveNode(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SendResource(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetResource(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ReleaseResource(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_ConnectionGrpcServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'GetNodeStructures': grpc.unary_unary_rpc_method_handler(
                    servicer.GetNodeStructures,
                    request_deserializer=connection__pb2.Empty.FromString,
                    response_serializer=connection__pb2.GetNodeStructuresResponse.SerializeToString,
            ),
            'GetCurrentExistingNodes': grpc.unary_unary_rpc_method_handler(
                    servicer.GetCurrentExistingNodes,
                    request_deserializer=connection__pb2.Empty.FromString,
                    response_serializer=connection__pb2.GetCurrentExistingNodesResponse.SerializeToString,
            ),
            'CreateNodeInstance': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateNodeInstance,
                    request_deserializer=connection__pb2.CreateNodeInstanceRequest.FromString,
                    response_serializer=connection__pb2.CreateNodeInstanceResponse.SerializeToString,
            ),
            'UpdatePortValues': grpc.unary_unary_rpc_method_handler(
                    servicer.UpdatePortValues,
                    request_deserializer=connection__pb2.UpdatePortValuesRequest.FromString,
                    response_serializer=connection__pb2.UpdatePortValuesResponse.SerializeToString,
            ),
            'ExecuteNode': grpc.unary_unary_rpc_method_handler(
                    servicer.ExecuteNode,
                    request_deserializer=connection__pb2.ExecuteNodeRequest.FromString,
                    response_serializer=connection__pb2.ExecuteNodeResponse.SerializeToString,
            ),
            'RemoveNode': grpc.unary_unary_rpc_method_handler(
                    servicer.RemoveNode,
                    request_deserializer=connection__pb2.RemoveNodeRequest.FromString,
                    response_serializer=connection__pb2.RemoveNodeResponse.SerializeToString,
            ),
            'SendResource': grpc.unary_unary_rpc_method_handler(
                    servicer.SendResource,
                    request_deserializer=connection__pb2.SendResourceRequest.FromString,
                    response_serializer=connection__pb2.SendResourceResponse.SerializeToString,
            ),
            'GetResource': grpc.unary_unary_rpc_method_handler(
                    servicer.GetResource,
                    request_deserializer=connection__pb2.GetResourceRequest.FromString,
                    response_serializer=connection__pb2.GetResourceResponse.SerializeToString,
            ),
            'ReleaseResource': grpc.unary_unary_rpc_method_handler(
                    servicer.ReleaseResource,
                    request_deserializer=connection__pb2.ReleaseResourceRequest.FromString,
                    response_serializer=connection__pb2.ReleaseResourceResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'connection.ConnectionGrpc', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class ConnectionGrpc(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def GetNodeStructures(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/connection.ConnectionGrpc/GetNodeStructures',
            connection__pb2.Empty.SerializeToString,
            connection__pb2.GetNodeStructuresResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetCurrentExistingNodes(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/connection.ConnectionGrpc/GetCurrentExistingNodes',
            connection__pb2.Empty.SerializeToString,
            connection__pb2.GetCurrentExistingNodesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateNodeInstance(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/connection.ConnectionGrpc/CreateNodeInstance',
            connection__pb2.CreateNodeInstanceRequest.SerializeToString,
            connection__pb2.CreateNodeInstanceResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UpdatePortValues(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/connection.ConnectionGrpc/UpdatePortValues',
            connection__pb2.UpdatePortValuesRequest.SerializeToString,
            connection__pb2.UpdatePortValuesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ExecuteNode(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/connection.ConnectionGrpc/ExecuteNode',
            connection__pb2.ExecuteNodeRequest.SerializeToString,
            connection__pb2.ExecuteNodeResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RemoveNode(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/connection.ConnectionGrpc/RemoveNode',
            connection__pb2.RemoveNodeRequest.SerializeToString,
            connection__pb2.RemoveNodeResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SendResource(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/connection.ConnectionGrpc/SendResource',
            connection__pb2.SendResourceRequest.SerializeToString,
            connection__pb2.SendResourceResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetResource(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/connection.ConnectionGrpc/GetResource',
            connection__pb2.GetResourceRequest.SerializeToString,
            connection__pb2.GetResourceResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ReleaseResource(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/connection.ConnectionGrpc/ReleaseResource',
            connection__pb2.ReleaseResourceRequest.SerializeToString,
            connection__pb2.ReleaseResourceResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



import logging
import re
from abc import ABC, abstractclassmethod, abstractmethod

import cv2
import numpy


class Resource(object):
  """
  The Resource class contains various data related to resource

  :ivar id: Resource id
  :vartype id: str
  :ivar type: Resource type
  :vartype id: str
  :ivar data: Resource data. Type of this variable depends on the `type` variable
  :vartype data: any

  .. warning::
    | Currently only `cv2img` type is supported
    | `cv2img`: OpenCV image
  """
  def __init__(self, id_, type_, data) -> None:
    self.id = id_
    self.type = type_
    self.data = data


class ResourceManagerBase(ABC):
  _instance = None

  def __init__(self) -> None:
    super().__init__()

  @abstractclassmethod
  def getInstance(cls) -> 'ResourceManagerBase':
    ...


class ResourceManager(ResourceManagerBase):
  """
  Resource manager class. Using this class resources can be accessed.
  """
  TYPE_CV2_IMAGE = 'cv2img'
  _instance = None

  def __init__(self) -> None:
    self._resources = {}
    self._logger = logging.getLogger(__name__.split('.')[0])

  @classmethod
  def getInstance(cls) -> 'ResourceManager':
    """
    Returns resource manager instance
    """
    if cls._instance is None:
      cls._instance = ResourceManager()
    return cls._instance

  @property
  def resources(self) -> 'dict[str, Resource]':
    """
    Returns resources dictionary

    :return: resources
    :rtype: dict[str, Resource]
    """
    return self._resources

  def createResource(self, data, type_: str, id_: str = None, overwrite: bool = False) -> str:
    """
    Creates resource.

    :param data: resource data
    :type data: any
    :param type_: resource type
    :type type_: str
    :param id_: resource id. If it's None id will be generated
    :type id_: str, optional
    :param overwrite: flag that indicates whether an existing resource will be overwritten
    :type overwrite: bool, optional

    :return: created resource id
    :rtype: str
    """
    if data is None:
      raise ValueError('Data is None')
    if id_ and not self.isResourceId(id_):
      raise ValueError(f'Create resource failed, Invalid id: {id_}')
    resId = 'res_{}'.format(
        self._generateUniqueString(8)) if id_ is None else id_
    self._logger.debug(f'Creating resource with id: {id_}')
    if self._isIdExists(resId):
      if not overwrite:
        raise ValueError(f'Create resource failed, id: {id_} already exist')
    if type_ == self.TYPE_CV2_IMAGE:
      self._resources[resId] = Resource(resId, type_, data)
      return resId
    else:
      self._logger.error(f'Currently, resource "{type_}" not supported')
    return None

  def createResourceFromBytes(self, data: bytes, type_: str, id_: str = None, overwrite: bool = False):
    """
    Creates resource from bytes.

    :param data: resource data
    :type data: bytes
    :param type_: resource type
    :type type_: str
    :param id_: resource id. If it's None id will be generated
    :type id_: str, optional
    :param overwrite: flag that indicates whether an existing resource will be overwritten
    :type overwrite: bool, optional

    :return: created resource id
    :rtype: str
    """
    if data is None:
      raise ValueError('Data is None')
    if id_ and not self.isResourceId(id_):
      raise ValueError(f'Create resource failed, Invalid id: {id_}')
    resId = f'res_{self._generateUniqueString(8)}' if id_ is None else id_
    if self._isIdExists(resId):
      if not overwrite:
        # raise ValueError(f'Create resource failed, id: {id_} already exist')
        return resId
    if type_ == self.TYPE_CV2_IMAGE:
      img = cv2.imdecode(numpy.frombuffer(data, numpy.uint8), cv2.IMREAD_COLOR)
      self._resources[resId] = Resource(resId, type_, img)
      return resId
    else:
      self._logger.error(f'Currently, resource "{type_}" not supported')
    return None

  def getResource(self, id_: str) -> 'tuple[str, str, object]':
    """
    Returns resource data

    :param id_: resource id
    :type id_: str

    :return: Resource. Tuple of {id, type, data}. Data depends on the resource type
    :rtype: tuple[str, str, object]
    """
    res = self._resources.get(id_, None)
    if res:
      return res.data
    return None

  def getResourceBytes(self, id_: str) -> 'tuple[str, str, bytes]':
    """
    Returns resource data in bytes

    :param id_: resource id
    :type id_: str

    :return: Resource. Tuple of {id, type, databytes}
    :rtype: tuple[str, str, bytes]
    """
    res = self._resources.get(id_, None)
    if res:
      is_success, im_buf_arr = cv2.imencode(".png", res.data)
      if is_success:
        return (res.id, res.type, im_buf_arr.tobytes())
    return None

  def removeResource(self, id_: str) -> bool:
    """
    Removes resource from resource list

    :param id_: resource id
    :type id_: str

    :return: result
    :rtype: bool
    """
    if id_ not in self._resources:
      self._logger.error(f'Resource "{id_}" does not exist')
      return False
    del self._resources[id_]
    return True

  def isResourceId(self, id):
    """
    Verifies resource id

    :param id: resource id
    :type id: str

    :return: result. True if id is valid, False if id is invalid
    :rtype: bool
    """
    return id and re.match('res_[a-zA-Z0-9]{8}$', id) is not None

  def _generateUniqueString(self, length):
    import uuid
    return uuid.uuid4().hex[:length].upper()

  def _isIdExists(self, id_):
    if id_ in self._resources:
      return True
    return False



import logging
from collections import OrderedDict

from ..resource import ResourceManager
from .constants import *
from .exceptions import PortNotFound
from .ports import (ActionNodeInput, ActionNodeOutput, ImageNodeInput, ImageNodeOutput,
                    NodeInputBase, NodeLocalBase, NodeOutputBase)
from .views import NodeViewBase


class NodeMeta(type):
  @classmethod
  def __prepare__(cls, name, bases):
    return OrderedDict()

  def __new__(cls, name, bases, clsdict):
    clsobj = super().__new__(cls, name, bases, dict(clsdict))
    inputs = getattr(clsobj, '_node_inputs_').copy() if hasattr(clsobj, '_node_inputs_') else OrderedDict()
    outputs = getattr(clsobj, '_node_outputs_').copy() if hasattr(clsobj, '_node_outputs_') else OrderedDict()
    locals = getattr(clsobj, '_locals_').copy() if hasattr(clsobj, '_locals_') else OrderedDict()
    views = getattr(clsobj, '_node_views').copy() if hasattr(clsobj, '_node_views') else {}
    for key, val in clsdict.items():
      if isinstance(val, NodeInputBase):
        val.key = key
        inputs[key] = val
      elif isinstance(val, NodeOutputBase):
        val.key = key
        outputs[key] = val
      elif isinstance(val, NodeLocalBase):
        val.key = key
        locals[key] = val
      elif isinstance(val, NodeViewBase):
        views[key] = val
    setattr(clsobj, '_node_inputs_', inputs)
    setattr(clsobj, '_node_outputs_', outputs)
    setattr(clsobj, '_locals_', locals)
    setattr(clsobj, '_node_views', views)
    return clsobj


class NodeBase(metaclass=NodeMeta):
  """
  The NodeBase class is the base class for the nodes
  To create external node, need to inherit this class and implement :func:`perform <mh_en_exec.nodes.NodeBase.perform>` function
  """
  NODE_NAME = 'Node Base'
  __package_path__ = 'external_node.data'
  __type__ = NODE_TYPE_DATA
  __can_be_public__ = False
  __disable__ = False

  def __init__(self, id_, name=NODE_NAME) -> None:
    super().__init__()
    self._logger = logging.getLogger(__name__.split('.')[0])
    self._id = id_
    self._name = name
    self._node_inputs = OrderedDict()
    self._node_outputs = OrderedDict()
    self._node_locals = OrderedDict()
    self._views = {}
    self._changed_values = {}
    for name, inpt in self._node_inputs_.items():
      self._addInput(name, inpt.copy())
    for name, outpt in self._node_outputs_.items():
      self._addOutput(name, outpt.copy())
    for name, local in self._locals_.items():
      self._addLocal(name, local.copy())
    for name, view in self._node_views.items():
      self._addView(name, view.copy())

  @classmethod
  def getNodePath(cls):
    return cls.__package_path__ + '.' + cls.__name__

  @classmethod
  def getNodeStructure(cls):
    return dict(
      classname=cls.__name__,
      name=cls.NODE_NAME,
      identifier=cls.getNodePath(),
      type=cls.__type__,
      is_public=cls.__can_be_public__,
      is_disabled=cls.__disable__,
      inputs=dict((k, v.toDict()) for k, v in cls._node_inputs_.items()),
      outputs=dict((k, v.toDict()) for k, v in cls._node_outputs_.items()),
      locals=dict((k, v.toDict()) for k, v in cls._locals_.items()),
      views=dict((k, v.toDict()) for k, v in cls._node_views.items())
    )

  @property
  def id(self) -> str:
    return self._id

  @id.setter
  def id(self, value):
    self._id = value

  @property
  def name(self) -> str:
    return self._name

  @name.setter
  def name(self, value):
    self._name = value

  def perform(self, *args, **kwargs) -> tuple:
    """
    | This function is called when a node is executed on the Machine Heads side.
    | Input port values will be set as arguments of this function.
    | Output port values need to be set as a return values of this function.
    | Return type:
    | In case of data node: `tuple[{output port values}]`
    | In case of action node: `bool, tuple[{output port values}]`
    | *first value of the action node return values is a bool value. If `True` `out_on_success` port will be called, if `False` `out_on_failure` port will be called*

    :raises NotImplementedError: In case of function is not overrode
    """
    raise NotImplementedError()

  def updatePortValues(self, ports):
    for key, value in ports.items():
      port = self._node_locals.get(key, None)
      if not port:
        port = self._node_inputs.get(key, None)
      if not port:
        port = self._node_outputs.get(key, None)
      if not port:
        # raise PortNotFound(f'Port {key} does not exist')
        self._logger.error(f'Port {key} does not exist')
        continue
      if value:
        port.deserialize(value)
      else:
        port.value = value

  def execute(self, portName, inputPorts):
    outputs = None
    inputArgs = []
    performFunction = self.perform
    self._logger.debug(f'execite function called. portName: {portName}, inputPorts: {inputPorts}')
    if portName:
      inputPort = self._node_inputs.get(portName, None)
      if inputPort and isinstance(inputPort, ActionNodeInput):
        inputArgs = inputPort.args
        performFunction = getattr(self, inputPort.functionName)
    inputsCount = len([inp for inp in self._node_inputs.values() if not isinstance(inp, ActionNodeInput)])
    inputs = self._prepareInputs(inputPorts)
    if len(inputs) != inputsCount:
      raise ValueError(f'Expected {inputsCount} input parameters, but there is {len(inputs)} in node {self.name}')
    try:
      self._logger.debug(f'Calling perform function: {performFunction}, args: {inputArgs}, kwargs: {inputs}')
      outputs = performFunction(*inputArgs, **inputs)
      self._logger.debug(f'Perform function called: {performFunction}')
    except NotImplementedError:
      return None
    except Exception as e:
      self._logger.exception('Node execution failed')
    self._logger.debug(f'_prepareOutputs: {outputs}')
    result = self._prepareOutputs(outputs)
    return result

  def getSerializedLocals(self) -> dict:
    result = {}
    for key, value in self._node_locals.items():
      result[key] = value.serialize()
    return result

  def _prepareInputs(self, values):
    if not values:
      return {}
    result = {}
    resourceManager = ResourceManager.getInstance()
    self._logger.debug('Preparing inputs...')
    self._logger.debug(f'values: {values}')
    for key, val in values.items():
      self._logger.debug(f'key: {key}, val: {val}')
      if val is not None:
        if isinstance(self._node_inputs[key], ImageNodeInput):
          self._logger.debug(f'{self._node_inputs[key]} is an instance of the ImageNodeInput')
          resource = resourceManager.getResource(val)
          if resource:
            result[key] = resource[2]
          else:
            result[key] = None
        else:
          self._logger.debug(f'{self._node_inputs[key]} is not an instance of the ImageNodeInput')
          result[key] = self._node_inputs[key].deserialize(val)
      elif key in self._node_inputs:
        result[key] = self._node_inputs[key].defaultValue
      else:
        result[key] = None
    return result

  def _prepareOutputs(self, values):
    if not isinstance(values, tuple):
      values = (values,)
    self._logger.debug('Preparing outputs...')
    result = {}
    resourceManager = ResourceManager.getInstance()
    keys = list(self._node_outputs.keys())
    if values is not None and len(values) != len(keys):
      raise ValueError(f'Expected {len(keys)} output parameters, but there is {len(values)} in node {self.name}')
    for i, key in enumerate(keys):
      if isinstance(self._node_outputs[key], ImageNodeOutput):
        self._logger.debug(f'{self._node_outputs[key]} is an instance of the ImageNodeOutput')
        img = values[i]
        if img is not None:
          resId = resourceManager.createResource(img, ResourceManager.TYPE_CV2_IMAGE)
          self._node_outputs[key].value = resId
      else:
        self._logger.debug(f'{self._node_outputs[key]} is not an instance of the ImageNodeOutput')
        self._node_outputs[key].value = values[i] if values is not None else None
      result[key] = self._node_outputs[key].serialize()
    return result

  def _addInput(self, name, input):
    if not hasattr(input, 'key'):
      input.key = name
    input.onValueChanged = self._onPortValueChanged
    setattr(self, name, input)
    self._node_inputs[name] = input

  def _addOutput(self, name, output):
    if not hasattr(output, 'key'):
      output.key = name
    output.onValueChanged = self._onPortValueChanged
    setattr(self, name, output)
    self._node_outputs[name] = output

  def _addLocal(self, name, local):
    if not hasattr(local, 'key'):
      local.key = name
    local.onValueChanged = self._onPortValueChanged
    setattr(self, name, local)
    self._node_locals[name] = local

  def _addView(self, name, view):
    self._views[name] = view

  def _onPortValueChanged(self, key, value):
    self._changed_values[key] = value


class ActionNodeBase(NodeBase):
  """
  The ActionNodeBase class is the base class for the action nodes
  To create external action node, need to inherit this class and implement :func:`perform <mh_en_exec.nodes.ActionNodeBase.perform>` function
  """
  __package_path__ = 'external_node.actions'
  __type__ = NODE_TYPE_ACTION

  in_action = ActionNodeInput()
  out_on_success = ActionNodeOutput()
  out_on_failure = ActionNodeOutput()

  def _prepareOutputs(self, outputs):
    self._logger.debug('Preparing outputs...')
    if outputs is None:
      return None
    result = {}
    resourceManager = ResourceManager.getInstance()
    data_keys = [key for key, port in self._node_outputs.items() if not isinstance(port, ActionNodeOutput)]
    action_keys = [key for key, port in self._node_outputs.items() if isinstance(port, ActionNodeOutput)]
    node_result, values = outputs
    if node_result:
      port = self._node_outputs.get(self.out_on_success.key, None)
      if port:
        port.value.result = True
    else:
      port = self._node_outputs.get(self.out_on_failure.key, None)
      if port:
        port.value.result = True
    if values is not None and len(values) != len(data_keys):
      raise ValueError(f'Expected {len(data_keys)} output parameters, but there is {len(values)} in node {self.name}')
    for i, key in enumerate(data_keys):
      if isinstance(self._node_outputs[key], ImageNodeOutput):
        self._logger.debug(f'{self._node_outputs[key]} is an instance of the ImageNodeOutput')
        img = values[i]
        if img is not None:
          resId = resourceManager.createResource(img, ResourceManager.TYPE_CV2_IMAGE)
          port = self._node_outputs[key]
          port.value = resId
          result[key] = port.serialize()
      else:
        self._logger.debug(f'{self._node_outputs[key]} is not an instance of the ImageNodeOutput')
        port = self._node_outputs[key]
        port.value = values[i] if values is not None else None
        result[key] = port.serialize()
    for key in action_keys:
      port = self._node_outputs[key]
      result[key] = port.serialize()
      port.value.result = False
    return result



from collections import OrderedDict
from copy import copy

from mh_en_exec.resource import ResourceManager

from .common import PointWithUnit, RectWithUnit, VectorWithUnit


class Descriptor:
  def __init__(self, name=None):
    self.name = name

  def __set__(self, instance, value):
    instance.__dict__[self.name] = value

  def __get__(self, instance, type=None):
    if instance is None:
      return None
    return instance.__dict__[self.name]

  def __delete__(self, instance):
    raise AttributeError("Can't delete")


class Any():
  pass


class Action(object):
  def __init__(self, result=False):
    self.result = result


class Typed(Descriptor):
  ty = object

  def __set__(self, instance, value):
    if value is not None and not isinstance(value, self.ty) and self.ty != Any:
      import traceback
      traceback.print_stack()
      raise TypeError('Expected %s' % self.ty)
    super().__set__(instance, value)

  def __serialize__(self, instance):
    if instance is None:
      return None
    if instance.__dict__[self.name] is None:
      return None
    if self.ty in (int, float, bool, str, list, Any):
      return copy(instance.__dict__[self.name])
    else:
      return copy(instance.__dict__[self.name].__dict__)

  def __deserialize__(self, instance, value):
    if instance is None:
      return None
    if self.ty in (int, float, bool, str, Any):
      instance.__dict__[self.name] = value
    else:
      if instance.__dict__[self.name] is None:
        instance.__dict__[self.name] = self.ty()
      for key, val in value.items():
        instance.__dict__[self.name].__dict__[key] = val
    return instance.__dict__[self.name]


class NodeLinkMeta(type):
  @classmethod
  def __prepare__(cls, name, bases):
    return OrderedDict()

  def __new__(cls, name, bases, clsdict):
    valueKey = None
    fields = []
    for key, val in clsdict.items():
      if isinstance(val, Descriptor):
        clsdict[key].name = key
        valueKey = key
        fields.append(key)
    clsobj = super().__new__(cls, name, bases, dict(clsdict))

    def _serialize(self):
      return clsdict['_value'].__serialize__(self)

    def _deserialize(self, value):
      return clsdict['_value'].__deserialize__(self, value)
    setattr(clsobj, 'serialize', _serialize)
    setattr(clsobj, 'deserialize', _deserialize)
    if valueKey:
      setattr(clsobj, '_value_type', clsdict[valueKey].ty)
    return clsobj


class NodeLink(object, metaclass=NodeLinkMeta):
  _value = Typed()

  def __init__(self, defaultValue=None):
    self._value = None
    if defaultValue is not None:
      if isinstance(defaultValue, dict):
        self._value = self.deserialize(defaultValue)
      else:
        self._value = defaultValue
    self.defaultValue = self.serialize()
    self.port = None

  def copy(self):
    obj = self.__class__(self.defaultValue)
    if hasattr(self, 'key'):
      obj.key = self.key
    return obj

  @property
  def value(self):
    """
    Port value property
    """
    return self._value

  @value.setter
  def value(self, value):
    self._value = value
    if hasattr(self, 'key'):
      self.onValueChanged(self.key, self.serialize())

  def toDict(self):
    return dict(
        key=getattr(self, 'key', ''),
        default_value=self.defaultValue,
        type_name=self.__class__.__name__
    )

  def onValueChanged(self, key, value):
    raise NotImplementedError()


class NodeInputBase(NodeLink):
  pass


class NodeOutputBase(NodeLink):
  pass


class NodeLocalBase(NodeLink):
  pass


class Cv2Image(Typed):
  ty = str


class AnyType(Typed):
  ty = Any


class StringType(Typed):
  ty = str


class ListType(Typed):
  ty = list


class Boolean(Typed):
  ty = bool


class Integer(Typed):
  ty = int


class Float(Typed):
  ty = float


class RectangleType(Typed):
  ty = RectWithUnit


class ActionType(Typed):
  ty = Action


class PointType(Typed):
  ty = PointWithUnit


class PointsType(Typed):
  ty = list


class VectorType(Typed):
  ty = VectorWithUnit

  def _extractValueKeys(self) -> tuple:
    return (('x', Float), ('y', Float), ('unit', StringType), ('magnitude', Float))

  def __serialize__(self, instance):
    if instance.__dict__[self.name] is None:
      return None
    v = instance.__dict__[self.name]
    return {'x': v.x, 'y': v.y, 'unit': v.unit, 'magnitude': v.magnitude}

  def __deserialize__(self, instance, value):
    instance.__dict__[self.name] = VectorWithUnit(value['x'], value['y'], value['unit'])
    return instance.__dict__[self.name]


class BooleanNodeInput(NodeInputBase):
  """
  Boolean node input port class
  """
  _value = Boolean()

  @NodeLink.value.getter
  def value(self):
    """
    Port value property

    :return: value
    :rtype: bool
    """
    return self._value


class BooleanNodeOutput(NodeOutputBase):
  """
  Boolean node output port class
  """
  _value = Boolean()

  @NodeLink.value.getter
  def value(self):
    """
    Port value property

    :return: value
    :rtype: bool
    """
    return self._value


class BooleanNodeLocal(NodeLocalBase):
  """
  Boolean node local port class
  """
  _value = Boolean()

  @NodeLink.value.getter
  def value(self):
    """
    Port value property

    :return: value
    :rtype: bool
    """
    return self._value


class IntegerNodeInput(NodeInputBase):
  """
  Integer node input port class
  """
  _value = Integer()

  @NodeLink.value.getter
  def value(self):
    """
    Port value property

    :return: value
    :rtype: int
    """
    return self._value


class IntegerNodeOutput(NodeOutputBase):
  """
  Integer node output port class
  """
  _value = Integer()

  @NodeLink.value.getter
  def value(self):
    """
    Port value property

    :return: value
    :rtype: int
    """
    return self._value


class IntegerNodeLocal(NodeLocalBase):
  """
  Integer node local port class
  """
  _value = Integer()

  @NodeLink.value.getter
  def value(self):
    """
    Port value property

    :return: value
    :rtype: int
    """
    return self._value


class FloatNodeInput(NodeInputBase):
  """
  Float node input port class
  """
  _value = Float()

  @NodeLink.value.getter
  def value(self):
    """
    Port value property

    :return: value
    :rtype: float
    """
    return self._value


class FloatNodeOutput(NodeOutputBase):
  """
  Float node output port class
  """
  _value = Float()

  @NodeLink.value.getter
  def value(self):
    """
    Port value property

    :return: value
    :rtype: float
    """
    return self._value


class FloatNodeLocal(NodeLocalBase):
  """
  Float node local port class
  """
  _value = Float()

  @NodeLink.value.getter
  def value(self):
    """
    Port value property

    :return: value
    :rtype: float
    """
    return self._value


class ImageNodeInput(NodeInputBase):
  """
  Image node input port class
  """
  _value = Cv2Image()

  @NodeLink.value.getter
  def value(self):
    """
    Port value property

    :return: image id
    :rtype: str
    """
    return self._value


class ImageNodeOutput(NodeOutputBase):
  """
  Image node output port class
  """
  _value = Cv2Image()

  @NodeLink.value.getter
  def value(self):
    """
    Port value property

    :return: image id
    :rtype: str
    """
    return self._value


class ImageNodeLocal(NodeLocalBase):
  """
  Image node local port class
  """
  _value = Cv2Image()

  @NodeLink.value.getter
  def value(self):
    """
    Port value property

    :return: OpenCV image
    :rtype: Mat
    """
    resourceManager = ResourceManager.getInstance()
    res = resourceManager.getResource(self._value)
    if res is not None:
      return res[2]
    return None


class RectangleNodeInput(NodeInputBase):
  """
  Rectangle node input port class
  """
  _value = RectangleType()

  @NodeLink.value.getter
  def value(self):
    """
    Port value property

    :return: rectangle
    :rtype: RectWithUnit
    """
    return self._value


class RectangleNodeOutput(NodeOutputBase):
  """
  Rectangle node output port class
  """
  _value = RectangleType()

  @NodeLink.value.getter
  def value(self):
    """
    Port value property

    :return: rectangle
    :rtype: RectWithUnit
    """
    return self._value


class RectangleNodeLocal(NodeLocalBase):
  """
  Rectangle node local port class
  """
  _value = RectangleType()

  @NodeLink.value.getter
  def value(self):
    """
    Port value property

    :return: rectangle
    :rtype: RectWithUnit
    """
    return self._value


class PointNodeInput(NodeInputBase):
  """
  Point node input port class
  """
  _value = PointType()

  @NodeLink.value.getter
  def value(self):
    """
    Port value property

    :return: point
    :rtype: PointWithUnit
    """
    return self._value


class PointNodeOutput(NodeOutputBase):
  """
  Point node output port class
  """
  _value = PointType()

  @NodeLink.value.getter
  def value(self):
    """
    Port value property

    :return: point
    :rtype: PointWithUnit
    """
    return self._value


class PointNodeLocal(NodeLocalBase):
  """
  Point node local port class
  """
  _value = PointType()

  @NodeLink.value.getter
  def value(self):
    """
    Port value property

    :return: point
    :rtype: PointWithUnit
    """
    return self._value


class AnyNodeInput(NodeInputBase):
  """
  Any node input port class
  """
  _value = AnyType()

  @NodeLink.value.getter
  def value(self):
    """
    Port value property

    :return: value
    :rtype: any
    """
    return self._value


class AnyNodeOutput(NodeOutputBase):
  """
  Any node output port class
  """
  _value = AnyType()

  @NodeLink.value.getter
  def value(self):
    """
    Port value property

    :return: value
    :rtype: any
    """
    return self._value


class ActionNodeInput(NodeInputBase):
  """
  Action node input port class
  """
  _value = ActionType()

  def __init__(self, functionName='perform', args=[], defaultValue=Action()):
    super().__init__(defaultValue=defaultValue)
    self.functionName = functionName
    self.args = args

  def copy(self):
    return self.__class__(self.functionName, self.args, self.defaultValue)

  def toDict(self):
    result = super().toDict()
    result['functionName'] = self.functionName
    result['args'] = self.args
    return result

  @NodeLink.value.getter
  def value(self):
    """
    Port value property

    :return: value
    :rtype: Action
    """
    return self._value


class ActionNodeOutput(NodeOutputBase):
  """
  Action node output port class
  """
  _value = ActionType()

  def __init__(self, defaultValue=Action()):
    super().__init__(defaultValue=defaultValue)

  def copy(self):
    return self.__class__(self.defaultValue)

  @NodeLink.value.getter
  def value(self):
    """
    Port value property

    :return: value
    :rtype: Action
    """
    return self._value


class StringNodeLocal(NodeLocalBase):
  """
  String node local port class
  """
  _value = StringType()

  @NodeLink.value.getter
  def value(self):
    """
    Port value property

    :return: value
    :rtype: str
    """
    return self._value


class StringNodeInput(NodeInputBase):
  """
  String node input port class
  """
  _value = StringType()

  @NodeLink.value.getter
  def value(self):
    """
    Port value property

    :return: value
    :rtype: str
    """
    return self._value


class StringNodeOutput(NodeOutputBase):
  """
  String node output port class
  """
  _value = StringType()

  @NodeLink.value.getter
  def value(self):
    """
    Port value property

    :return: value
    :rtype: str
    """
    return self._value


class VectorNodeInput(NodeInputBase):
  """
  Vector node input port class
  """
  _value = VectorType()

  @NodeLink.value.getter
  def value(self):
    """
    Port value property

    :return: vector
    :rtype: VectorWithUnit
    """
    return self._value


class VectorNodeOutput(NodeOutputBase):
  """
  Vector node output port class
  """
  _value = VectorType()

  @NodeLink.value.getter
  def value(self):
    """
    Port value property

    :return: vector
    :rtype: VectorWithUnit
    """
    return self._value


class VectorNodeLocal(NodeLocalBase):
  """
  Vector node local port class
  """
  _value = VectorType()

  @NodeLink.value.getter
  def value(self):
    """
    Port value property

    :return: vector
    :rtype: VectorWithUnit
    """
    return self._value


class ListNodeLocal(NodeLocalBase):
  _value = ListType()


class ListNodeInput(NodeInputBase):
  _value = ListType()


class ListNodeOutput(NodeOutputBase):
  _value = ListType()



import math


class Size(object):
  """
  The Size class defines the size of a two-dimensional object using floating point precision.

  :param width: width
  :type width: float, optional
  :param height: height
  :type height: float, optional
  """
  def __init__(self, width=0, height=0):
    self.width = width
    self.height = height

  def __str__(self):
    return '{{width: {0}, height: {1}}}'.format(self.width, self.height)

  def __add__(self, other):
    return Size(self.width + other.width, self.height + other.height)

  def __sub__(self, other):
    return Size(self.width - other.width, self.height - other.height)

  def __eq__(self, other):
    return self.width == other.width and self.height == other.height


class Point(object):
  """
  The Point class defines the point in two-dimensional space using floating point precision.

  :param x: coordinate on the x-axis
  :type x: float, optional
  :param y: coordinate on the y-axis
  :type y: float, optional
  """
  def __init__(self, x=0, y=0):
    self.x = x
    self.y = y

  def __str__(self):
    return '{{x: {0}, y: {1}}}'.format(self.x, self.y)

  def __add__(self, other):
    return Point(self.x + other.x, self.y + other.y)

  def __sub__(self, other):
    return Point(self.x - other.x, self.y - other.y)

  def __eq__(self, other):
    if other is None:
      return False
    return self.x == other.x and self.y == other.y


class PointWithUnit(Point):
  """
  The PointWithUnit class defines the point in two-dimensional space with particular unit using floating point precision.

  :param x: coordinate on the x-axis
  :type x: float, optional
  :param y: coordinate on the y-axis
  :type y: float, optional
  :param unit: coordinate unit
  :type unit: str, optional

  .. note::
    | *Coordinate Unit:*
    | 'px' - value in pixels
    | 'hu' - value in height units.
    | Ex. 1hu is `{height of the screen} / 100`
    | '%' - value in percentages.
    | Ex. 50% of the x is half of the width of the screen
  """

  POINT_UNIT_PX = 'px'
  POINT_UNIT_PER = '%'
  POINT_UNIT_HU = 'hu'

  def __init__(self, x=0, y=0, unit=POINT_UNIT_PX):
    super().__init__(x=x, y=y)
    self.unit = unit

  def __str__(self):
    return '{{x: {0}{2}, y: {1}{2}}}'.format(self.x, self.y, self.unit)

  def __add__(self, other):
    if self.unit != other.unit:
      raise ValueError('units of points is not equal')
    return PointWithUnit(self.x + other.x, self.y + other.y, self.unit)

  def __sub__(self, other):
    if self.unit != other.unit:
      raise ValueError('units of points is not equal')
    return PointWithUnit(self.x - other.x, self.y - other.y, self.unit)

  def __eq__(self, other):
    if other is None:
      return False
    return super().__eq__(other) and self.unit == other.unit

  @classmethod
  def lerp(cls, start, end, t):
    """
    Linearly interpolates between two points

    :param start: Start point
    :type start: PointWithUnit
    :param end: End point
    :type end: PointWithUnit
    :param t: value used to interpolate between `start` and `end`. Range between 0.0 and 1.0
    :type t: float
    """
    if start.unit != end.unit:
      raise ValueError('units of points is not equal')
    return PointWithUnit(start.x * (1.0 - t) + end.x * t, start.y * (1.0 - t) + end.y * t, start.unit)

  def toPx(self, size):
    """
    Returns PointWithUnit in pixel units

    :param size: Screen size
    :type size: Size

    :raises ValueError: In case of unknown unit value

    :return: point in pixel units
    :rtype: PointWithUnit
    """
    if size is None:
      return None
    if self.unit == self.POINT_UNIT_PX:
      return PointWithUnit(self.x, self.y, self.unit)
    elif self.unit == self.POINT_UNIT_PER:
      return PointWithUnit(size.width * (self.x / 100.0), size.height * (self.y / 100.0), self.POINT_UNIT_PX)
    elif self.unit == self.POINT_UNIT_HU:
      pxinhu = size.height / 100.0
      return PointWithUnit(self.x * pxinhu, self.y * pxinhu, self.POINT_UNIT_PX)
    else:
      raise ValueError('Unknown unit. unit: \"{}\"'.format(self.unit))


class Rect(object):
  """
  The Rect class defines the rectangle shape in two-dimensional space using floating point precision.

  :param left: leftmost coordinate on the x-axis
  :type left: float, optional
  :param top: topmost coordinate on the y-axis
  :type top: float, optional
  :param right: rightmost coordinate on the x-axis
  :type right: float, optional
  :param bottom: bottommost coordinate on the y-axis
  :type bottom: float, optional
  """
  def __init__(self, left=0, top=0, right=0, bottom=0):
    self.left = left
    self.top = top
    self.right = right
    self.bottom = bottom

  def center(self):
    """
    Returns rectangle's center point

    :return: rectangle center
    :rtype: Point
    """
    size = self.size()
    return Point(self.left + (size.width / 2.0), self.top + (size.height / 2.0))

  def size(self):
    """
    Returns rectangle size

    :return: size
    :rtype: Size
    """
    return Size(self.right - self.left, self.bottom - self.top)

  def isInside(self, point):
    """
    Checks if the point is inside the rectangle

    :param point: point
    :type point: Point

    :return: True if inside and False if outside
    :rtype: bool
    """
    return point.x > self.left and point.x < self.right and point.y > self.top and point.y < self.bottom

  def isValid(self):
    """
    Checks if the rectangle is valid

    :return: True if valid and False if invalid
    :rtype: bool
    """
    if self.left == 0 and self.top == 0 and self.right == 0 and self.bottom == 0:
      return False
    if self.right <= self.left or self.bottom <= self.top:
      return False
    return True

  def getAxisSwapped(self):
    """
    Returns axis swapped rectangle

    :return: axis swapped rectangle
    :rtype: Rect
    """
    return Rect(self.top, self.left, self.bottom, self.right)

  def __add__(self, other):
    return Rect(self.left + other.left, self.top + other.top, self.right + other.right, self.bottom + other.bottom)

  def __sub__(self, other):
    return Rect(self.left - other.left, self.top - other.top, self.right - other.right, self.bottom - other.bottom)

  def __str__(self):
    return 'left: {}, top: {}, right: {}, bottom: {}'.format(self.left, self.top, self.right, self.bottom)

  def __mul__(self, other):
    if not isinstance(other, int) and not isinstance(other, float):
      raise NotImplementedError()
    return Rect(self.left * other, self.top * other, self.right * other, self.bottom * other)

  __rmul__ = __mul__


class RectWithUnit(Rect):
  """
  The RectWithUnit class defines the rectangle shape in two-dimensional space with particular unit using floating point precision.

  :param left: leftmost coordinate on the x-axis
  :type left: float, optional
  :param top: topmost coordinate on the y-axis
  :type top: float, optional
  :param right: rightmost coordinate on the x-axis
  :type right: float, optional
  :param bottom: bottommost coordinate on the y-axis
  :type bottom: float, optional
  :param unit: coordinate unit
  :type unit: str, optional

  .. note::
    | *Coordinate Unit:*
    | 'px' - value in pixels
    | 'hu' - value in height units.
    | Ex. 1hu is `{height of the screen} / 100`
    | '%' - value in percentages.
    | Ex. 50% of the x is half of the width of the screen
  """
  RECT_UNIT_PX = 'px'
  RECT_UNIT_PER = '%'
  RECT_UNIT_HU = 'hu'

  def __init__(self, left=0, top=0, right=0, bottom=0, unit=RECT_UNIT_PX):
    super().__init__(left, top, right, bottom)
    self.unit = unit

  def center(self):
    """
    Returns rectangle's center point

    :return: rectangle center
    :rtype: PointWithUnit
    """
    point = super().center()
    return PointWithUnit(point.x, point.y, self.unit)

  def isInside(self, point):
    """
    Checks if the point is inside the rectangle

    :param point: point
    :type point: PointWithUnit

    :raises ValueError: In case of unit value of the point is not the same as unit of the rectangle

    :return: True if inside and False if outside
    :rtype: bool
    """
    if hasattr(point, 'unit'):
      if point.unit != self.unit:
        raise ValueError(
            'Rect({}) and point({}) have different units'.format(self.unit, point.unit))
    else:
      if self.unit != self.RECT_UNIT_PX:
        raise ValueError('Rect({}) and point(px) have different units')
    return super().isInside(point)

  def isValid(self):
    """
    Checks if the rectangle is valid

    :return: True if valid and False if invalid
    :rtype: bool
    """
    if self.left == 0 and self.top == 0 and self.right == 0 and self.bottom == 0:
      return False
    if self.right <= self.left or self.bottom <= self.top:
      return False
    return True

  def getAxisSwapped(self):
    """
    Returns axis swapped rectangle

    :return: axis swapped rectangle
    :rtype: RectWithUnit
    """
    return RectWithUnit(self.top, self.left, self.bottom, self.right, self.unit)

  def __add__(self, other):
    if self.unit != other.unit:
      raise ValueError('units of rect is not equal')
    return RectWithUnit(self.left + other.left, self.top + other.top, self.right + other.right, self.bottom + other.bottom, self.unit)

  def __sub__(self, other):
    if self.unit != other.unit:
      raise ValueError('units of rect is not equal')
    return RectWithUnit(self.left - other.left, self.top - other.top, self.right - other.right, self.bottom - other.bottom, self.unit)

  def __str__(self):
    return 'left: {}, top: {}, right: {}, bottom: {}, unit: {}'.format(self.left, self.top, self.right, self.bottom, self.unit)

  def __mul__(self, other):
    if not isinstance(other, int) and not isinstance(other, float):
      raise NotImplementedError()
    return RectWithUnit(self.left * other, self.top * other, self.right * other, self.bottom * other, self.unit)

  def __eq__(self, other):
    if other is None:
      return False
    return self.unit == other.unit and self.left == other.left and self.top == other.top and self.right == other.right and self.bottom == other.bottom

  def toPx(self, size):
    """
    Returns RectWithUnit in pixel units

    :param size: Screen size
    :type size: Size

    :raises ValueError: In case of unknown unit value

    :return: rectangle in pixel units
    :rtype: RectWithUnit
    """
    if size is None:
      return None
    if not isinstance(size, Size) and not isinstance(size, tuple):
      raise ValueError('Invalid parameter. Expected instance of Size or tuple')
    w = size.width if isinstance(size, Size) else size[0]
    h = size.height if isinstance(size, Size) else size[1]
    if self.unit == self.RECT_UNIT_PX:
      return RectWithUnit(self.left, self.top, self.right, self.bottom, self.unit)
    elif self.unit == self.RECT_UNIT_PER:
      return RectWithUnit(w * (self.left / 100.0), h * (self.top / 100.0), w * (self.right / 100.0), h * (self.bottom / 100.0), self.RECT_UNIT_PX)
    elif self.unit == self.RECT_UNIT_HU:
      pxinhu = h / 100.0
      return RectWithUnit(self.left * pxinhu, self.top * pxinhu, self.right * pxinhu, self.bottom * pxinhu, self.RECT_UNIT_PX)
    else:
      raise ValueError('Unknown unit. unit: \"{}\"'.format(self.unit))

  def transform(self, delta, relSize=None):
    """
    Transforms rectangle to delta

    :param delta: delta rectangle
    :type delta: RectWithUnit
    :param relSize: Screen size
    :type relSize: Size

    :raises ValueError: In case of delta is not instance of RectWithUnit or relSize is not set in case of this rectangle is not in pixel unit

    :return: transformed rectangle
    :rtype: RectWithUnit
    """
    if delta is None:
      return None
    if not isinstance(delta, RectWithUnit):
      raise ValueError(
          'Invalid parameter. Expected instance of \"RectWithUnit\" class')
    size = self.size()
    deltaSize = delta.size()

    res = None
    if delta.unit == self.RECT_UNIT_PX:
      if self.unit != self.RECT_UNIT_PX and relSize is None:
        raise ValueError('Invalid parameter. Relative size is not specified')
      res = (self if self.unit == self.RECT_UNIT_PX else self.toPx(relSize)) + delta
    elif delta.unit == self.RECT_UNIT_PER:
      res = RectWithUnit()
      res.left = self.left + (size.width * (delta.left / 100.0))
      res.top = self.top + (size.height * (delta.top / 100.0))
      res.right = res.left + (size.width * (deltaSize.width / 100.0))
      res.bottom = res.top + (size.height * (deltaSize.height / 100.0))
      res.unit = self.unit
    elif delta.unit == self.RECT_UNIT_HU:
      huval = size.height / 100.0
      res = RectWithUnit()
      res.left = self.left + (delta.left * huval)
      res.top = self.top + (delta.top * huval)
      res.right = res.left + (size.width + (deltaSize.width * huval))
      res.bottom = res.top + (size.height + (deltaSize.height * huval))
      res.unit = self.unit
    return res

    # if self.unit == self.RECT_UNIT_PX:
    #   if delta.unit == self.RECT_UNIT_PX:
    #     return self + delta
    #   elif delta.unit == self.RECT_UNIT_PER:
    #     return RectWithUnit(self.left + (size.width * (delta.left / 100.0)),      self.top + (size.height * (delta.top / 100.0)),
    #                         self.left + (size.width * (deltaSize.width / 100.0)), self.top + (size.height * (deltaSize.height / 100.0)),
    #                         self.RECT_UNIT_PX)
    #   elif delta.unit == self.RECT_UNIT_HU:
    #     pxinhu = size.height / 100.0
    #     return RectWithUnit(self.left + (delta.left * pxinhu), self.top + (delta.top * pxinhu),
    #                         self.left + (size.width * (deltaSize.width * pxinhu)), self.top + (size.height * (deltaSize.height * pxinhu)), self.RECT_UNIT_PX)
    # elif self.unit == self.RECT_UNIT_PER:
    #   if delta.unit == self.RECT_UNIT_PX:
    #     return self.toPx(relSize) + delta
    #   elif delta.unit == self.RECT_UNIT_PER:
    #     return RectWithUnit(self.left + (size.width * (delta.left / 100.0)),      self.top + (size.height * (delta.top / 100.0)),
    #                         self.left + (size.width * (deltaSize.width / 100.0)), self.top + (size.height * (deltaSize.height / 100.0)),
    #                         self.RECT_UNIT_PER)
    #   elif delta.unit == self.RECT_UNIT_HU:
    #     perinhu = size.height / 100.0
    #     return RectWithUnit(self.left + (delta.left * pxinhu), self.top + (delta.top * pxinhu),
    #                         self.left + (size.width * (deltaSize.width * pxinhu)), self.top + (size.height * (deltaSize.height * pxinhu)), self.RECT_UNIT_PER)
    # elif self.unit == self.RECT_UNIT_HU:
    #   if delta.unit == self.RECT_UNIT_PX:
    #     if relSize is None:
    #       raise ValueError('Invalid parameter. Relative size is not specified')
    #     return self.toPx(relSize) + delta
    #   elif delta.unit == self.RECT_UNIT_PER:
    #     return RectWithUnit(self.left + (size.width * (delta.left / 100.0)),      self.top + (size.height * (delta.top / 100.0)),
    #                         self.left + (size.width * (deltaSize.width / 100.0)), self.top + (size.height * (deltaSize.height / 100.0)),
    #                         self.RECT_UNIT_HU)
    #   elif delta.unit == self.RECT_UNIT_HU:
    #     perinhu = size.height / 100.0
    #     return RectWithUnit(self.left + (delta.left * pxinhu), self.top + (delta.top * pxinhu),
    #                         self.left + (size.width * (deltaSize.width * pxinhu)), self.top + (size.height * (deltaSize.height * pxinhu)), self.RECT_UNIT_HU)
#
    # return None

  __rmul__ = __mul__


class Vector(object):
  """
  The Vector class defines the vector in two-dimensional space using floating point precision.

  :param x: coordinate on the x-axis
  :type x: float, optional
  :param y: coordinate on the y-axis
  :type y: float, optional
  """
  def __init__(self, x=0.0, y=0.0):
    super().__init__()
    self.x = x
    self.y = y

  @classmethod
  def fromPoint(cls, point: Point) -> 'Vector':
    """
    Returns Vector converted from point

    :raises ValueError: If the point does not exist

    :param point: Point in two-dimensional space
    :type point: Point
    """
    if point is None:
      raise ValueError('point does not exist')
    return cls(point.x, point.y)

  @classmethod
  def fromPoints(cls, origin: Point, point: Point) -> 'Vector':
    """
    Returns Vector calculated from two points

    :raises ValueError: If origin or point does not exist

    :param origin: Starting point in two-dimensional space
    :type origin: Point
    :param point: Terminal point in two-dimensional space
    :type point: Point
    """
    if origin is None:
      raise ValueError('origin does not exist')
    if point is None:
      raise ValueError('point does not exist')
    return cls(point.x - origin.x, point.y - origin.y)

  @property
  def magnitude(self):
    """
    Returns the length of this vector

    :return: length of this vector
    :rtype: float
    """
    return math.sqrt(self.x * self.x + self.y * self.y)

  def normalize(self):
    """
    Makes this vector have a magnitude of 1.0
    """
    length = math.fabs(math.sqrt((self.x * self.x) + (self.y * self.y)))
    self.x = self.x / length
    self.y = self.y / length

  def __str__(self):
    return '{{x: {0}, y: {1}}}'.format(self.x, self.y)

  def __add__(self, other: 'Vector'):
    return Vector(self.x + other.x, self.y + other.y)

  def __sub__(self, other: 'Vector'):
    return Vector(self.x - other.x, self.y - other.y)

  def __eq__(self, other: 'Vector'):
    if other is None:
      return False
    return self.x == other.x and self.y == other.y

  def __mul__(self, other):
    if not isinstance(other, int) and not isinstance(other, float):
      raise NotImplementedError()
    return Vector(self.x * other, self.y * other)

  __rmul__ = __mul__

  def __truediv__(self, other):
    if not isinstance(other, int) and not isinstance(other, float):
      raise NotImplementedError()
    return Vector(self.x / other, self.y / other)


class VectorWithUnit(Vector):
  """
  The VectorWithUnit class defines the vector in two-dimensional space with particular unit using floating point precision.

  :param x: coordinate on the x-axis
  :type x: float, optional
  :param y: coordinate on the y-axis
  :type y: float, optional
  :param unit: coordinate unit
  :type unit: str, optional

  .. note::
    | *Coordinate Unit:*
    | 'px' - value in pixels
    | 'hu' - value in height units.
    | Ex. 1hu is `{height of the screen} / 100`
    | '%' - value in percentages.
    | Ex. 50% of the x is half of the width of the screen
  """
  VECTOR_UNIT_PX = 'px'
  VECTOR_UNIT_PER = '%'
  VECTOR_UNIT_HU = 'hu'

  def __init__(self, x=0.0, y=0.0, unit=VECTOR_UNIT_PX, **kwargs):
    super().__init__(x, y)
    self.unit = unit

  @classmethod
  def fromPoint(cls, point: PointWithUnit) -> 'VectorWithUnit':
    """
    Returns VectorWithUnit converted from point

    :raises ValueError: If the point does not exist

    :param point: Point in two-dimensional space
    :type point: PointWithUnit
    """
    if point is None:
      raise ValueError('point does not exist')
    return cls(point.x, point.y, point.unit)

  @classmethod
  def fromPoints(cls, origin: PointWithUnit, point: PointWithUnit) -> 'VectorWithUnit':
    """
    Returns VectorWithUnit calculated from two points

    :raises ValueError: If starting point or terminal point does not exist, also if unit of two points is not same

    :param origin: Starting point in two-dimensional space
    :type origin: PointWithUnit
    :param point: Terminal point in two-dimensional space
    :type point: PointWithUnit
    """
    if origin is None:
      raise ValueError('origin does not exist')
    if point is None:
      raise ValueError('point does not exist')
    if origin.unit != point.unit:
      raise ValueError('units of points is not equal')
    return cls(point.x - origin.x, point.y - origin.y, origin.unit)

  def __str__(self):
    return '{{x: {0}, y: {1}, unit: {2}}}'.format(self.x, self.y, self.unit)

  def __add__(self, other: 'VectorWithUnit'):
    if self.unit != other.unit:
      raise ValueError('units of vectors is not equal')
    return VectorWithUnit(self.x + other.x, self.y + other.y, self.unit)

  def __sub__(self, other: 'VectorWithUnit'):
    if self.unit != other.unit:
      raise ValueError('units of vectors is not equal')
    return VectorWithUnit(self.x - other.x, self.y - other.y, self.unit)

  def __eq__(self, other: 'VectorWithUnit'):
    if other is None:
      return False
    return self.x == other.x and self.y == other.y and self.unit == other.unit

  def __mul__(self, other):
    if not isinstance(other, int) and not isinstance(other, float):
      raise NotImplementedError()
    return VectorWithUnit(self.x * other, self.y * other, self.unit)

  __rmul__ = __mul__

  def __truediv__(self, other):
    if not isinstance(other, int) and not isinstance(other, float):
      raise NotImplementedError()
    return VectorWithUnit(self.x / other, self.y / other, self.unit)



from .main import _createServer

_server = None


def start(nodes, port=50051):
  """
  Start external node server

  :param nodes: list of node classes
  :type nodes: list[NodeBase]
  :param port: server port number
  :type port: int
  """
  global _server
  _server = _createServer(nodes, port)
  _server.start()


def exec(nodes, port=50051):
  """
  Start external node server and wait for termination

  :param nodes: list of node classes
  :type nodes: list[NodeBase]
  :param port: server port number
  :type port: int
  """
  global _server
  _server = _createServer(nodes, port)
  _server.start()
  _server.wait_for_termination()



from abc import ABC, abstractmethod
import logging


class ConnectionListenerBase(ABC):
  @abstractmethod
  def onGetNodesStructures(self) -> list:
    ...

  @abstractmethod
  def onGetCurrentNodes(self) -> dict:
    ...

  @abstractmethod
  def onCreateNode(self, id_, path, name) -> 'tuple[bool, str]':
    ...

  @abstractmethod
  def onUpdatePortValues(self, id_, values) -> 'tuple[bool, str]':
    ...

  @abstractmethod
  def onNodeExecution(self, id_, portName, values) -> 'tuple[bool, str, dict, dict]':
    ...

  @abstractmethod
  def onRemoveNode(self, id_) -> 'tuple[bool, str]':
    ...


class ConnectionResourceListenerBase(ABC):
  @abstractmethod
  def onResourceReceive(self, id_, type_, data) -> bool:
    ...

  @abstractmethod
  def onGetResource(self, id_, remove) -> 'tuple[str, str, bytes]':
    ...

  @abstractmethod
  def onReleaseResource(self, id_) -> 'tuple[bool, str]':
    ...


class ConnectionBase(object):
  def __init__(self, eventListener=None, resourceListener=None):
    self._eventListener = eventListener
    self._resourceListener = resourceListener
    self._logger = logging.getLogger(__name__.split('.')[0])

  @property
  def eventListener(self) -> ConnectionListenerBase:
    return self._eventListener

  @eventListener.setter
  def eventListener(self, value):
    self._eventListener = value

  @property
  def resourceListener(self) -> ConnectionResourceListenerBase:
    return self._resourceListener

  @resourceListener.setter
  def resourceListener(self, value):
    self._resourceListener = value

  def _getNodesStructuresEmit(self) -> list:
    if self.eventListener:
      return self.eventListener.onGetNodesStructures()
    return None

  def _getCurrentExistingNodesEmit(self) -> dict:
    if self.eventListener:
      return self.eventListener.onGetCurrentNodes()
    return None

  def _createNodeEmit(self, id_, path, name):
    if self.eventListener:
      return self.eventListener.onCreateNode(id_, path, name)
    errorMessage = 'Event listener not set'
    self._logger.error(errorMessage)
    return False, errorMessage

  def _updatePortValues(self, id_, values):
    if self.eventListener:
      return self.eventListener.onUpdatePortValues(id_, values)
    errorMessage = 'Event listener not set'
    self._logger.error(errorMessage)
    return False, errorMessage

  def _nodeExecutionEmit(self, id_, portName, values):
    if self.eventListener:
      return self.eventListener.onNodeExecution(id_, portName, values)
    errorMessage = 'Event listener not set'
    self._logger.error(errorMessage)
    return False, errorMessage, None, None

  def _removeNodeEmit(self, id_):
    if self.eventListener:
      return self.eventListener.onRemoveNode(id_)
    errorMessage = 'Event listener not set'
    self._logger.error(errorMessage)
    return False, errorMessage

  def _resourceReceivedEmit(self, id_, type_, data):
    if self.resourceListener:
      return self.resourceListener.onResourceReceive(id_, type_, data)
    return None

  def _getResourceEmit(self, id_, remove):
    if self.resourceListener:
      return self.resourceListener.onGetResource(id_, remove)
    return None

  def _releaseResourceEmit(self, id_):
    if self.resourceListener:
      return self.resourceListener.onReleaseResource(id_)
    errorMessage = 'Event listener not set'
    self._logger.error(errorMessage)
    return False, errorMessage



class NodeViewBase(object):
  def __init__(self, linkName: str, label: str):
    self.linkName = linkName
    self.label = label

  def copy(self) -> 'NodeViewBase':
    return self.__class__(self.linkName, self.label)

  def toDict(self) -> dict:
    return dict(class_name=self.__class__.__name__, linkName=self.linkName, label=self.label)


class NodeCheckboxView(NodeViewBase):
  """
  The class for UI representation of a port that contains boolean value

  .. image:: img/views/checkbox_view.png

  *node that contains this view*

  :param linkName: The name of the boolean port for which this object will be the UI representation
  :type linkName: str
  :param label: The text that will be displayed on the node
  :type label: str, optional
  """
  def __init__(self, linkName: str, label: str = 'Value'):
    super().__init__(linkName, label)


class NodeComboboxView(NodeViewBase):
  """
  The class for the UI representation of the port and gives the ability to choose from several predefined values

  .. image:: img/views/combobox_view.png

  *node that contains this view*

  :param linkName: The name of the port for which this object will be the UI representation
  :type linkName: str
  :param label: The text that will be displayed on the node
  :type label: str
  :param values: list of the values that will be diplayed on the node
  :type values: list
  """
  def __init__(self, linkName: str, label: str, values: list):
    super().__init__(linkName, label)
    self.values = values

  def copy(self) -> 'NodeComboboxView':
    return self.__class__(self.linkName, self.label, self.values)

  def toDict(self) -> dict:
    value = super().toDict()
    value['values'] = self.values
    return value


class NodeFloatView(NodeViewBase):
  """
  The class for UI representation of a port that contains float value

  .. image:: img/views/float_view.png

  *node that contains this view*

  :param linkName: The name of the float port for which this object will be the UI representation
  :type linkName: str
  :param label: The text that will be displayed on the node
  :type label: str, optional
  """
  def __init__(self, linkName: str, label: str = 'Value'):
    super().__init__(linkName, label)


class NodeImageView(NodeViewBase):
  """
  The class for UI representation of a port that contains image value

  .. image:: img/views/image_view.png

  *node that contains this view*

  :param linkName: The name of the image port for which this object will be the UI representation
  :type linkName: str
  :param label: The text that will be displayed on the node
  :type label: str, optional
  """
  def __init__(self, linkName: str, label: str = 'Image'):
    super().__init__(linkName, label)


class NodeIntegerView(NodeViewBase):
  """
  The class for UI representation of a port that contains integer value

  .. image:: img/views/integer_view.png

  *node that contains this view*

  :param linkName: The name of the integer port for which this object will be the UI representation
  :type linkName: str
  :param label: The text that will be displayed on the node
  :type label: str, optional
  :param readonly: The flag that determines whether the value can be changed by user
  :type readonly: bool, optional
  :param rangeTop: Maximum possible value
  :type rangeTop: int, optional
  :param rangeBottom: Minimum possible value
  :type rangeBottom: int, optional
  """
  def __init__(self, linkName: str, label: str = 'Value', readonly: bool = False, rangeTop: int = None, rangeBottom: int = None):
    super().__init__(linkName, label)
    self.readonly = readonly
    self.rangeTop = rangeTop
    self.rangeBottom = rangeBottom

  def copy(self) -> 'NodeIntegerView':
    return self.__class__(self.linkName, self.label, self.readonly, self.rangeTop, self.rangeBottom)

  def toDict(self) -> dict:
    value = super().toDict()
    value['readonly'] = self.readonly
    value['rangeTop'] = self.rangeTop
    value['rangeBottom'] = self.rangeBottom
    return value


class NodePairComboView(NodeViewBase):
  """
  The class for the UI representation of the port and gives the ability to choose from several predefined values

  .. image:: img/views/pair_combobox_view.png

  *node that contains this view*

  :param linkName: The name of the port for which this object will be the UI representation
  :type linkName: str
  :param label: The text that will be displayed on the node
  :type label: str
  :param items: dict of the values that will be displayed on the node. The key of the dict will be set to port as value and the value of the dict will be displayed on the node.
  :type items: dict
  """
  def __init__(self, linkName: str, label: str, items: dict):
    super().__init__(linkName, label)
    self.items = items

  def copy(self) -> 'NodePairComboView':
    return self.__class__(self.linkName, self.label, self.items)

  def toDict(self) -> dict:
    value = super().toDict()
    value['items'] = self.items
    return value


class NodePointView(NodeViewBase):
  """
  The class for UI representation of a port that contains :class:`PointWithUnit <mh_en_exec.nodes.common.PointWithUnit>` value

  .. image:: img/views/point_view.png

  *node that contains this view*

  :param linkName: The name of the point port for which this object will be the UI representation
  :type linkName: str
  :param label: The text that will be displayed on the node
  :type label: str, optional
  """
  def __init__(self, linkName: str, label: str = 'Point'):
    super().__init__(linkName, label)


class NodeRectView(NodeViewBase):
  """
  The class for UI representation of a port that contains :class:`RectWithUnit <mh_en_exec.nodes.common.RectWithUnit>` value

  .. image:: img/views/rect_view.png

  *node that contains this view*

  :param linkName: The name of the rect port for which this object will be the UI representation
  :type linkName: str
  :param label: The text that will be displayed on the node
  :type label: str, optional
  """
  def __init__(self, linkName: str, label: str = 'Rect'):
    super().__init__(linkName, label)


class NodeStringView(NodeViewBase):
  """
  The class for UI representation of a port that contains string value

  .. image:: img/views/string_view.png

  *node that contains this view*

  :param linkName: The name of the string port for which this object will be the UI representation
  :type linkName: str
  :param label: The text that will be displayed on the node
  :type label: str, optional
  """
  def __init__(self, linkName: str, label: str = 'Value'):
    super().__init__(linkName, label)


class NodeTextBoxView(NodeViewBase):
  """
  The class for UI representation of a port that contains multiline string value

  .. image:: img/views/textbox_view.png

  *node that contains this view*

  :param linkName: The name of the string port for which this object will be the UI representation
  :type linkName: str
  :param label: The text that will be displayed on the node
  :type label: str, optional
  """
  def __init__(self, linkName: str, label: str = 'Text', hint: str = None):
    super().__init__(linkName, label)
    self.hint = hint

  def copy(self) -> 'NodeTextBoxView':
    return self.__class__(self.linkName, self.label, self.hint)

  def toDict(self) -> dict:
    value = super().toDict()
    value['hint'] = self.hint
    return value


class NodeVectorView(NodeViewBase):
  """
  The class for UI representation of a port that contains :class:`VectorWithUnit <mh_en_exec.nodes.common.VectorWithUnit>` value

  .. image:: img/views/vector_view.png

  *node that contains this view*

  :param linkName: The name of the vector port for which this object will be the UI representation
  :type linkName: str
  :param label: The text that will be displayed on the node
  :type label: str, optional
  """
  def __init__(self, linkName: str, label: str = 'Vector'):
    super().__init__(linkName, label)



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
  long_description = fh.read()

setup(
  name='mh-en-exec',
  version='1.0.0',
  description='Package for executing machine heads external nodes',
  long_description=long_description,
  long_description_content_type="text/markdown",
  url='https://mheads.net/external_nodes',
  author='Opus',
  author_email='mh@opus.co.jp',
  license='MIT',
  package_dir={"": "src"},
  packages=['mh_en_exec', 'mh_en_exec.connection', 'mh_en_exec.nodes', 'mh_en_exec.nodes.views', 'mh_en_exec.resource'],
  zip_safe=False,
  keywords=['automation', 'test', 'machine heads', 'custom', 'node'],
  install_requires=[
      'grpcio>=1.37.0',
      'grpcio-tools>=1.37.0',
      'opencv-python>=4.1.2.30'
  ],
  classifiers=[
      'Development Status :: 5 - Production/Stable',
      'License :: OSI Approved :: MIT License',
      'Programming Language :: Python :: 3.6',
      'Operating System :: OS Independent'
  ],
  python_requires=">=3.6"
)



import logging

from .exceptions import NodeNotFoundError, PortNotFound
from .node_base import NodeBase
from .node_factory import NodeFactory


class NodeManager(object):
  _instance = None

  def __init__(self) -> None:
    super().__init__()
    self._factory = NodeFactory()
    self._nodes = {}
    self._logger = logging.getLogger(__name__.split('.')[0])

  @property
  def nodes(self) -> 'dict[str, NodeBase]':
    return self._nodes

  @classmethod
  def getInstance(cls) -> 'NodeManager':
    if cls._instance is None:
      cls._instance = NodeManager()
    return cls._instance

  def registerNodes(self, nodes):
    for node in nodes:
      self._factory.registerNode(node)

  def getNodesStructures(self) -> list:
    return self._factory.getNodesStructures()

  def createNodeInstance(self, id_, path, name) -> 'tuple[bool, str]':
    self._logger.debug(f'creating node[{path}] instance: {{id: {id_}, name: {name}}}')
    nodeClass = None
    try:
      nodeClass = self._factory.getNodeClass(path)
    except NodeNotFoundError as e:
      errorMessage = str(e)
      self._logger.error(errorMessage)
      return False, errorMessage
    if id_ in self._nodes:
      errorMessage = f'The node with the id "{id_}" already exists'
      self._logger.error(errorMessage)
      return False, errorMessage
    node = nodeClass(id_, name)
    self._nodes[id_] = node
    return True, None

  def updateNodePortValues(self, id_, ports) -> 'tuple[bool, str]':
    self._logger.debug(f'updating port values: {{id: {id_}, ports: {ports}}}')
    node = self._nodes.get(id_, None)
    if not node:
      errorMessage = f'The node with the id {id_} does not exist'
      self._logger.error(errorMessage)
      return False, errorMessage

    if ports:
      try:
        node.updatePortValues(ports)
      except PortNotFound as e:
        errorMessage = str(e)
        self._logger.error(errorMessage)
        return False, errorMessage
    return True, None

  def executeNode(self, id_, portName, values) -> 'tuple[bool, str, dict, dict]':
    self._logger.debug(f'executing node: {{id: {id_}, portName: {portName}, ports: {values}}}')
    node = self._nodes.get(id_, None)
    if not node:
      errorMessage = f'The node with the id {id_} does not exist'
      self._logger.error(errorMessage)
      return False, errorMessage, None, None

    try:
      outputs = node.execute(portName, values)
      locals = node.getSerializedLocals()
    except KeyboardInterrupt as e:
      raise e
    except Exception as e:
      errorMessage = str(e)
      # self._logger.error(errorMessage)
      self._logger.exception(errorMessage)
      return False, errorMessage, None, None
    return True, '', locals, outputs

  def removeNode(self, id_) -> 'tuple[bool, str]':
    self._logger.debug(f'removing node: {{id: {id_}}}')
    if id_ not in self._nodes:
      errorMessage = f'The node with the id {id_} does not exist'
      self._logger.error(errorMessage)
      return False, errorMessage

    del self._nodes[id_]
    return True, None



import logging
from .connection import (ConnectionGrpc, ConnectionListenerBase,
                         ConnectionResourceListenerBase)
from .nodes import NodeManager
from .resource import ResourceManager


class _ConnectionListener(ConnectionListenerBase):
  def __init__(self) -> None:
    self._nodesManager = NodeManager.getInstance()
    self._logger = logging.getLogger(__name__.split('.')[0])

  def onGetNodesStructures(self) -> list:
    self._logger.debug(f'onGetNodesStructures')
    return self._nodesManager.getNodesStructures()

  def onGetCurrentNodes(self) -> dict:
    result = {}
    for key, node in self._nodesManager.nodes.items():
      result[key] = node.getNodePath()
    return result

  def onCreateNode(self, id_, path, name) -> 'tuple[bool, str]':
    return self._nodesManager.createNodeInstance(id_, path, name)

  def onUpdatePortValues(self, id_, ports) -> 'tuple[bool, str]':
    return self._nodesManager.updateNodePortValues(id_, ports)

  def onNodeExecution(self, id_, portName, inputPorts) -> 'tuple[bool, str, dict, dict]':
    return self._nodesManager.executeNode(id_, portName, inputPorts)

  def onRemoveNode(self, id_) -> 'tuple[bool, str]':
    return self._nodesManager.removeNode(id_)


class _ConnectionResourceListener(ConnectionResourceListenerBase):
  def __init__(self) -> None:
    self._resourceManager = ResourceManager.getInstance()
    self._logger = logging.getLogger(__name__.split('.')[0])

  def onResourceReceive(self, id_, type_, data) -> bool:
    # print('onResourceReceive')
    # print(f'id: {id_}, type: {type_}, datalen: {len(data)}')
    self._logger.debug(f'onResourceReceive id: {id_}, type: {type_}')
    result = self._resourceManager.createResourceFromBytes(
        data, type_, id_=id_)
    # print(f'result: {result}')
    return result is not None

  def onGetResource(self, id_, remove) -> 'tuple[str, str, bytes]':
    # print('onGetResource')
    # print(f'id: {id_}, remove: {remove}')
    self._logger.debug(f'onGetResource id: {id_}, remove: {remove}')
    result = self._resourceManager.getResourceBytes(id_)
    if remove:
      self._resourceManager.removeResource(id_)
    return result

  def onReleaseResource(self, id_) -> 'tuple[bool, str]':
    self._logger.debug(f'onReleaseResource id: {id_}')
    if id_ not in self._resourceManager.resources:
      return False, 'Resource does not exist'
    self._resourceManager.removeResource(id_)
    return True, ''


def _createServer(nodes, port):
  logger = logging.getLogger(__name__.split('.')[0])
  ch = logging.StreamHandler()
  formatter = logging.Formatter(
      '%(asctime)s %(levelname)s [%(name)s] %(message)s')
  ch.setFormatter(formatter)
  logger.addHandler(ch)

  nodesManager = NodeManager.getInstance()
  nodesManager.registerNodes(nodes)

  logger.debug('creating grpc server')
  _listener = _ConnectionListener()
  _resourceListener = _ConnectionResourceListener()
  server = ConnectionGrpc.createServerInstance(
      _listener, _resourceListener, port=port)
  return server



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: connection.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='connection.proto',
  package='connection',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x10\x63onnection.proto\x12\nconnection\"\x07\n\x05\x45mpty\"?\n\x19GetNodeStructuresResponse\x12\x10\n\x08serverId\x18\x01 \x01(\t\x12\x10\n\x08jsonData\x18\x02 \x01(\t\"E\n\x1fGetCurrentExistingNodesResponse\x12\x10\n\x08serverId\x18\x01 \x01(\t\x12\x10\n\x08jsonData\x18\x02 \x01(\t\"G\n\x19\x43reateNodeInstanceRequest\x12\n\n\x02id\x18\x01 \x01(\t\x12\x10\n\x08nodePath\x18\x02 \x01(\t\x12\x0c\n\x04name\x18\x03 \x01(\t\"T\n\x1a\x43reateNodeInstanceResponse\x12\x10\n\x08serverId\x18\x01 \x01(\t\x12\x0e\n\x06result\x18\x02 \x01(\x08\x12\x14\n\x0c\x65rrorMessage\x18\x03 \x01(\t\"=\n\x17UpdatePortValuesRequest\x12\n\n\x02id\x18\x01 \x01(\t\x12\x16\n\x0eportValuesJson\x18\x02 \x01(\t\"R\n\x18UpdatePortValuesResponse\x12\x10\n\x08serverId\x18\x01 \x01(\t\x12\x0e\n\x06result\x18\x02 \x01(\x08\x12\x14\n\x0c\x65rrorMessage\x18\x03 \x01(\t\"O\n\x12\x45xecuteNodeRequest\x12\n\n\x02id\x18\x01 \x01(\t\x12\x10\n\x08portName\x18\x02 \x01(\t\x12\x1b\n\x13inputPortValuesJson\x18\x03 \x01(\t\"\x88\x01\n\x13\x45xecuteNodeResponse\x12\x10\n\x08serverId\x18\x01 \x01(\t\x12\x0e\n\x06result\x18\x02 \x01(\x08\x12\x14\n\x0c\x65rrorMessage\x18\x03 \x01(\t\x12\x1b\n\x13localPortValuesJson\x18\x04 \x01(\t\x12\x1c\n\x14outputPortValuesJson\x18\x05 \x01(\t\"\x1f\n\x11RemoveNodeRequest\x12\n\n\x02id\x18\x01 \x01(\t\"L\n\x12RemoveNodeResponse\x12\x10\n\x08serverId\x18\x01 \x01(\t\x12\x0e\n\x06result\x18\x02 \x01(\x08\x12\x14\n\x0c\x65rrorMessage\x18\x03 \x01(\t\",\n\x10ResourceMetaData\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04type\x18\x02 \x01(\t\"\x1f\n\x0cResourceData\x12\x0f\n\x07\x63ontent\x18\x01 \x01(\x0c\"m\n\x13SendResourceRequest\x12.\n\x08metadata\x18\x01 \x01(\x0b\x32\x1c.connection.ResourceMetaData\x12&\n\x04\x64\x61ta\x18\x02 \x01(\x0b\x32\x18.connection.ResourceData\"8\n\x14SendResourceResponse\x12\x10\n\x08serverId\x18\x01 \x01(\t\x12\x0e\n\x06result\x18\x02 \x01(\x08\"0\n\x12GetResourceRequest\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0e\n\x06remove\x18\x02 \x01(\x08\"\x7f\n\x13GetResourceResponse\x12\x10\n\x08serverId\x18\x01 \x01(\t\x12.\n\x08metadata\x18\x02 \x01(\x0b\x32\x1c.connection.ResourceMetaData\x12&\n\x04\x64\x61ta\x18\x03 \x01(\x0b\x32\x18.connection.ResourceData\"$\n\x16ReleaseResourceRequest\x12\n\n\x02id\x18\x01 \x01(\t\"Q\n\x17ReleaseResourceResponse\x12\x10\n\x08serverId\x18\x01 \x01(\t\x12\x0e\n\x06result\x18\x02 \x01(\x08\x12\x14\n\x0c\x65rrorMessage\x18\x03 \x01(\t2\xac\x06\n\x0e\x43onnectionGrpc\x12O\n\x11GetNodeStructures\x12\x11.connection.Empty\x1a%.connection.GetNodeStructuresResponse\"\x00\x12[\n\x17GetCurrentExistingNodes\x12\x11.connection.Empty\x1a+.connection.GetCurrentExistingNodesResponse\"\x00\x12\x65\n\x12\x43reateNodeInstance\x12%.connection.CreateNodeInstanceRequest\x1a&.connection.CreateNodeInstanceResponse\"\x00\x12_\n\x10UpdatePortValues\x12#.connection.UpdatePortValuesRequest\x1a$.connection.UpdatePortValuesResponse\"\x00\x12P\n\x0b\x45xecuteNode\x12\x1e.connection.ExecuteNodeRequest\x1a\x1f.connection.ExecuteNodeResponse\"\x00\x12M\n\nRemoveNode\x12\x1d.connection.RemoveNodeRequest\x1a\x1e.connection.RemoveNodeResponse\"\x00\x12S\n\x0cSendResource\x12\x1f.connection.SendResourceRequest\x1a .connection.SendResourceResponse\"\x00\x12P\n\x0bGetResource\x12\x1e.connection.GetResourceRequest\x1a\x1f.connection.GetResourceResponse\"\x00\x12\\\n\x0fReleaseResource\x12\".connection.ReleaseResourceRequest\x1a#.connection.ReleaseResourceResponse\"\x00\x62\x06proto3'
)




_EMPTY = _descriptor.Descriptor(
  name='Empty',
  full_name='connection.Empty',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=32,
  serialized_end=39,
)


_GETNODESTRUCTURESRESPONSE = _descriptor.Descriptor(
  name='GetNodeStructuresResponse',
  full_name='connection.GetNodeStructuresResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='serverId', full_name='connection.GetNodeStructuresResponse.serverId', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='jsonData', full_name='connection.GetNodeStructuresResponse.jsonData', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=41,
  serialized_end=104,
)


_GETCURRENTEXISTINGNODESRESPONSE = _descriptor.Descriptor(
  name='GetCurrentExistingNodesResponse',
  full_name='connection.GetCurrentExistingNodesResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='serverId', full_name='connection.GetCurrentExistingNodesResponse.serverId', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='jsonData', full_name='connection.GetCurrentExistingNodesResponse.jsonData', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=106,
  serialized_end=175,
)


_CREATENODEINSTANCEREQUEST = _descriptor.Descriptor(
  name='CreateNodeInstanceRequest',
  full_name='connection.CreateNodeInstanceRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='connection.CreateNodeInstanceRequest.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='nodePath', full_name='connection.CreateNodeInstanceRequest.nodePath', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name', full_name='connection.CreateNodeInstanceRequest.name', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=177,
  serialized_end=248,
)


_CREATENODEINSTANCERESPONSE = _descriptor.Descriptor(
  name='CreateNodeInstanceResponse',
  full_name='connection.CreateNodeInstanceResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='serverId', full_name='connection.CreateNodeInstanceResponse.serverId', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='result', full_name='connection.CreateNodeInstanceResponse.result', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='errorMessage', full_name='connection.CreateNodeInstanceResponse.errorMessage', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=250,
  serialized_end=334,
)


_UPDATEPORTVALUESREQUEST = _descriptor.Descriptor(
  name='UpdatePortValuesRequest',
  full_name='connection.UpdatePortValuesRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='connection.UpdatePortValuesRequest.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='portValuesJson', full_name='connection.UpdatePortValuesRequest.portValuesJson', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=336,
  serialized_end=397,
)


_UPDATEPORTVALUESRESPONSE = _descriptor.Descriptor(
  name='UpdatePortValuesResponse',
  full_name='connection.UpdatePortValuesResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='serverId', full_name='connection.UpdatePortValuesResponse.serverId', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='result', full_name='connection.UpdatePortValuesResponse.result', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='errorMessage', full_name='connection.UpdatePortValuesResponse.errorMessage', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=399,
  serialized_end=481,
)


_EXECUTENODEREQUEST = _descriptor.Descriptor(
  name='ExecuteNodeRequest',
  full_name='connection.ExecuteNodeRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='connection.ExecuteNodeRequest.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='portName', full_name='connection.ExecuteNodeRequest.portName', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='inputPortValuesJson', full_name='connection.ExecuteNodeRequest.inputPortValuesJson', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=483,
  serialized_end=562,
)


_EXECUTENODERESPONSE = _descriptor.Descriptor(
  name='ExecuteNodeResponse',
  full_name='connection.ExecuteNodeResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='serverId', full_name='connection.ExecuteNodeResponse.serverId', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='result', full_name='connection.ExecuteNodeResponse.result', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='errorMessage', full_name='connection.ExecuteNodeResponse.errorMessage', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='localPortValuesJson', full_name='connection.ExecuteNodeResponse.localPortValuesJson', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='outputPortValuesJson', full_name='connection.ExecuteNodeResponse.outputPortValuesJson', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=565,
  serialized_end=701,
)


_REMOVENODEREQUEST = _descriptor.Descriptor(
  name='RemoveNodeRequest',
  full_name='connection.RemoveNodeRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='connection.RemoveNodeRequest.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=703,
  serialized_end=734,
)


_REMOVENODERESPONSE = _descriptor.Descriptor(
  name='RemoveNodeResponse',
  full_name='connection.RemoveNodeResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='serverId', full_name='connection.RemoveNodeResponse.serverId', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='result', full_name='connection.RemoveNodeResponse.result', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='errorMessage', full_name='connection.RemoveNodeResponse.errorMessage', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=736,
  serialized_end=812,
)


_RESOURCEMETADATA = _descriptor.Descriptor(
  name='ResourceMetaData',
  full_name='connection.ResourceMetaData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='connection.ResourceMetaData.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='type', full_name='connection.ResourceMetaData.type', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=814,
  serialized_end=858,
)


_RESOURCEDATA = _descriptor.Descriptor(
  name='ResourceData',
  full_name='connection.ResourceData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='content', full_name='connection.ResourceData.content', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=860,
  serialized_end=891,
)


_SENDRESOURCEREQUEST = _descriptor.Descriptor(
  name='SendResourceRequest',
  full_name='connection.SendResourceRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='metadata', full_name='connection.SendResourceRequest.metadata', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='data', full_name='connection.SendResourceRequest.data', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=893,
  serialized_end=1002,
)


_SENDRESOURCERESPONSE = _descriptor.Descriptor(
  name='SendResourceResponse',
  full_name='connection.SendResourceResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='serverId', full_name='connection.SendResourceResponse.serverId', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='result', full_name='connection.SendResourceResponse.result', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1004,
  serialized_end=1060,
)


_GETRESOURCEREQUEST = _descriptor.Descriptor(
  name='GetResourceRequest',
  full_name='connection.GetResourceRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='connection.GetResourceRequest.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='remove', full_name='connection.GetResourceRequest.remove', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1062,
  serialized_end=1110,
)


_GETRESOURCERESPONSE = _descriptor.Descriptor(
  name='GetResourceResponse',
  full_name='connection.GetResourceResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='serverId', full_name='connection.GetResourceResponse.serverId', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='metadata', full_name='connection.GetResourceResponse.metadata', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='data', full_name='connection.GetResourceResponse.data', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1112,
  serialized_end=1239,
)


_RELEASERESOURCEREQUEST = _descriptor.Descriptor(
  name='ReleaseResourceRequest',
  full_name='connection.ReleaseResourceRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='connection.ReleaseResourceRequest.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1241,
  serialized_end=1277,
)


_RELEASERESOURCERESPONSE = _descriptor.Descriptor(
  name='ReleaseResourceResponse',
  full_name='connection.ReleaseResourceResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='serverId', full_name='connection.ReleaseResourceResponse.serverId', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='result', full_name='connection.ReleaseResourceResponse.result', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='errorMessage', full_name='connection.ReleaseResourceResponse.errorMessage', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1279,
  serialized_end=1360,
)

_SENDRESOURCEREQUEST.fields_by_name['metadata'].message_type = _RESOURCEMETADATA
_SENDRESOURCEREQUEST.fields_by_name['data'].message_type = _RESOURCEDATA
_GETRESOURCERESPONSE.fields_by_name['metadata'].message_type = _RESOURCEMETADATA
_GETRESOURCERESPONSE.fields_by_name['data'].message_type = _RESOURCEDATA
DESCRIPTOR.message_types_by_name['Empty'] = _EMPTY
DESCRIPTOR.message_types_by_name['GetNodeStructuresResponse'] = _GETNODESTRUCTURESRESPONSE
DESCRIPTOR.message_types_by_name['GetCurrentExistingNodesResponse'] = _GETCURRENTEXISTINGNODESRESPONSE
DESCRIPTOR.message_types_by_name['CreateNodeInstanceRequest'] = _CREATENODEINSTANCEREQUEST
DESCRIPTOR.message_types_by_name['CreateNodeInstanceResponse'] = _CREATENODEINSTANCERESPONSE
DESCRIPTOR.message_types_by_name['UpdatePortValuesRequest'] = _UPDATEPORTVALUESREQUEST
DESCRIPTOR.message_types_by_name['UpdatePortValuesResponse'] = _UPDATEPORTVALUESRESPONSE
DESCRIPTOR.message_types_by_name['ExecuteNodeRequest'] = _EXECUTENODEREQUEST
DESCRIPTOR.message_types_by_name['ExecuteNodeResponse'] = _EXECUTENODERESPONSE
DESCRIPTOR.message_types_by_name['RemoveNodeRequest'] = _REMOVENODEREQUEST
DESCRIPTOR.message_types_by_name['RemoveNodeResponse'] = _REMOVENODERESPONSE
DESCRIPTOR.message_types_by_name['ResourceMetaData'] = _RESOURCEMETADATA
DESCRIPTOR.message_types_by_name['ResourceData'] = _RESOURCEDATA
DESCRIPTOR.message_types_by_name['SendResourceRequest'] = _SENDRESOURCEREQUEST
DESCRIPTOR.message_types_by_name['SendResourceResponse'] = _SENDRESOURCERESPONSE
DESCRIPTOR.message_types_by_name['GetResourceRequest'] = _GETRESOURCEREQUEST
DESCRIPTOR.message_types_by_name['GetResourceResponse'] = _GETRESOURCERESPONSE
DESCRIPTOR.message_types_by_name['ReleaseResourceRequest'] = _RELEASERESOURCEREQUEST
DESCRIPTOR.message_types_by_name['ReleaseResourceResponse'] = _RELEASERESOURCERESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Empty = _reflection.GeneratedProtocolMessageType('Empty', (_message.Message,), {
  'DESCRIPTOR' : _EMPTY,
  '__module__' : 'connection_pb2'
  # @@protoc_insertion_point(class_scope:connection.Empty)
  })
_sym_db.RegisterMessage(Empty)

GetNodeStructuresResponse = _reflection.GeneratedProtocolMessageType('GetNodeStructuresResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETNODESTRUCTURESRESPONSE,
  '__module__' : 'connection_pb2'
  # @@protoc_insertion_point(class_scope:connection.GetNodeStructuresResponse)
  })
_sym_db.RegisterMessage(GetNodeStructuresResponse)

GetCurrentExistingNodesResponse = _reflection.GeneratedProtocolMessageType('GetCurrentExistingNodesResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETCURRENTEXISTINGNODESRESPONSE,
  '__module__' : 'connection_pb2'
  # @@protoc_insertion_point(class_scope:connection.GetCurrentExistingNodesResponse)
  })
_sym_db.RegisterMessage(GetCurrentExistingNodesResponse)

CreateNodeInstanceRequest = _reflection.GeneratedProtocolMessageType('CreateNodeInstanceRequest', (_message.Message,), {
  'DESCRIPTOR' : _CREATENODEINSTANCEREQUEST,
  '__module__' : 'connection_pb2'
  # @@protoc_insertion_point(class_scope:connection.CreateNodeInstanceRequest)
  })
_sym_db.RegisterMessage(CreateNodeInstanceRequest)

CreateNodeInstanceResponse = _reflection.GeneratedProtocolMessageType('CreateNodeInstanceResponse', (_message.Message,), {
  'DESCRIPTOR' : _CREATENODEINSTANCERESPONSE,
  '__module__' : 'connection_pb2'
  # @@protoc_insertion_point(class_scope:connection.CreateNodeInstanceResponse)
  })
_sym_db.RegisterMessage(CreateNodeInstanceResponse)

UpdatePortValuesRequest = _reflection.GeneratedProtocolMessageType('UpdatePortValuesRequest', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEPORTVALUESREQUEST,
  '__module__' : 'connection_pb2'
  # @@protoc_insertion_point(class_scope:connection.UpdatePortValuesRequest)
  })
_sym_db.RegisterMessage(UpdatePortValuesRequest)

UpdatePortValuesResponse = _reflection.GeneratedProtocolMessageType('UpdatePortValuesResponse', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEPORTVALUESRESPONSE,
  '__module__' : 'connection_pb2'
  # @@protoc_insertion_point(class_scope:connection.UpdatePortValuesResponse)
  })
_sym_db.RegisterMessage(UpdatePortValuesResponse)

ExecuteNodeRequest = _reflection.GeneratedProtocolMessageType('ExecuteNodeRequest', (_message.Message,), {
  'DESCRIPTOR' : _EXECUTENODEREQUEST,
  '__module__' : 'connection_pb2'
  # @@protoc_insertion_point(class_scope:connection.ExecuteNodeRequest)
  })
_sym_db.RegisterMessage(ExecuteNodeRequest)

ExecuteNodeResponse = _reflection.GeneratedProtocolMessageType('ExecuteNodeResponse', (_message.Message,), {
  'DESCRIPTOR' : _EXECUTENODERESPONSE,
  '__module__' : 'connection_pb2'
  # @@protoc_insertion_point(class_scope:connection.ExecuteNodeResponse)
  })
_sym_db.RegisterMessage(ExecuteNodeResponse)

RemoveNodeRequest = _reflection.GeneratedProtocolMessageType('RemoveNodeRequest', (_message.Message,), {
  'DESCRIPTOR' : _REMOVENODEREQUEST,
  '__module__' : 'connection_pb2'
  # @@protoc_insertion_point(class_scope:connection.RemoveNodeRequest)
  })
_sym_db.RegisterMessage(RemoveNodeRequest)

RemoveNodeResponse = _reflection.GeneratedProtocolMessageType('RemoveNodeResponse', (_message.Message,), {
  'DESCRIPTOR' : _REMOVENODERESPONSE,
  '__module__' : 'connection_pb2'
  # @@protoc_insertion_point(class_scope:connection.RemoveNodeResponse)
  })
_sym_db.RegisterMessage(RemoveNodeResponse)

ResourceMetaData = _reflection.GeneratedProtocolMessageType('ResourceMetaData', (_message.Message,), {
  'DESCRIPTOR' : _RESOURCEMETADATA,
  '__module__' : 'connection_pb2'
  # @@protoc_insertion_point(class_scope:connection.ResourceMetaData)
  })
_sym_db.RegisterMessage(ResourceMetaData)

ResourceData = _reflection.GeneratedProtocolMessageType('ResourceData', (_message.Message,), {
  'DESCRIPTOR' : _RESOURCEDATA,
  '__module__' : 'connection_pb2'
  # @@protoc_insertion_point(class_scope:connection.ResourceData)
  })
_sym_db.RegisterMessage(ResourceData)

SendResourceRequest = _reflection.GeneratedProtocolMessageType('SendResourceRequest', (_message.Message,), {
  'DESCRIPTOR' : _SENDRESOURCEREQUEST,
  '__module__' : 'connection_pb2'
  # @@protoc_insertion_point(class_scope:connection.SendResourceRequest)
  })
_sym_db.RegisterMessage(SendResourceRequest)

SendResourceResponse = _reflection.GeneratedProtocolMessageType('SendResourceResponse', (_message.Message,), {
  'DESCRIPTOR' : _SENDRESOURCERESPONSE,
  '__module__' : 'connection_pb2'
  # @@protoc_insertion_point(class_scope:connection.SendResourceResponse)
  })
_sym_db.RegisterMessage(SendResourceResponse)

GetResourceRequest = _reflection.GeneratedProtocolMessageType('GetResourceRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETRESOURCEREQUEST,
  '__module__' : 'connection_pb2'
  # @@protoc_insertion_point(class_scope:connection.GetResourceRequest)
  })
_sym_db.RegisterMessage(GetResourceRequest)

GetResourceResponse = _reflection.GeneratedProtocolMessageType('GetResourceResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETRESOURCERESPONSE,
  '__module__' : 'connection_pb2'
  # @@protoc_insertion_point(class_scope:connection.GetResourceResponse)
  })
_sym_db.RegisterMessage(GetResourceResponse)

ReleaseResourceRequest = _reflection.GeneratedProtocolMessageType('ReleaseResourceRequest', (_message.Message,), {
  'DESCRIPTOR' : _RELEASERESOURCEREQUEST,
  '__module__' : 'connection_pb2'
  # @@protoc_insertion_point(class_scope:connection.ReleaseResourceRequest)
  })
_sym_db.RegisterMessage(ReleaseResourceRequest)

ReleaseResourceResponse = _reflection.GeneratedProtocolMessageType('ReleaseResourceResponse', (_message.Message,), {
  'DESCRIPTOR' : _RELEASERESOURCERESPONSE,
  '__module__' : 'connection_pb2'
  # @@protoc_insertion_point(class_scope:connection.ReleaseResourceResponse)
  })
_sym_db.RegisterMessage(ReleaseResourceResponse)



_CONNECTIONGRPC = _descriptor.ServiceDescriptor(
  name='ConnectionGrpc',
  full_name='connection.ConnectionGrpc',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=1363,
  serialized_end=2175,
  methods=[
  _descriptor.MethodDescriptor(
    name='GetNodeStructures',
    full_name='connection.ConnectionGrpc.GetNodeStructures',
    index=0,
    containing_service=None,
    input_type=_EMPTY,
    output_type=_GETNODESTRUCTURESRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetCurrentExistingNodes',
    full_name='connection.ConnectionGrpc.GetCurrentExistingNodes',
    index=1,
    containing_service=None,
    input_type=_EMPTY,
    output_type=_GETCURRENTEXISTINGNODESRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='CreateNodeInstance',
    full_name='connection.ConnectionGrpc.CreateNodeInstance',
    index=2,
    containing_service=None,
    input_type=_CREATENODEINSTANCEREQUEST,
    output_type=_CREATENODEINSTANCERESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='UpdatePortValues',
    full_name='connection.ConnectionGrpc.UpdatePortValues',
    index=3,
    containing_service=None,
    input_type=_UPDATEPORTVALUESREQUEST,
    output_type=_UPDATEPORTVALUESRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ExecuteNode',
    full_name='connection.ConnectionGrpc.ExecuteNode',
    index=4,
    containing_service=None,
    input_type=_EXECUTENODEREQUEST,
    output_type=_EXECUTENODERESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='RemoveNode',
    full_name='connection.ConnectionGrpc.RemoveNode',
    index=5,
    containing_service=None,
    input_type=_REMOVENODEREQUEST,
    output_type=_REMOVENODERESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SendResource',
    full_name='connection.ConnectionGrpc.SendResource',
    index=6,
    containing_service=None,
    input_type=_SENDRESOURCEREQUEST,
    output_type=_SENDRESOURCERESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetResource',
    full_name='connection.ConnectionGrpc.GetResource',
    index=7,
    containing_service=None,
    input_type=_GETRESOURCEREQUEST,
    output_type=_GETRESOURCERESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ReleaseResource',
    full_name='connection.ConnectionGrpc.ReleaseResource',
    index=8,
    containing_service=None,
    input_type=_RELEASERESOURCEREQUEST,
    output_type=_RELEASERESOURCERESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_CONNECTIONGRPC)

DESCRIPTOR.services_by_name['ConnectionGrpc'] = _CONNECTIONGRPC

# @@protoc_insertion_point(module_scope)
